package org.helioviewer.base.math;

/**
 * An immutable Interval class, used for integer ranges.  Both end points are
 * considered to be included in the interval (open interval).
 * @author caplins
 *
 */
public class Interval {
	
	/** The left side of the interval.  Guaranteed that leftSide <= rightSide. */
	public final int leftSide;
	
	/** The right side of the interval.  Guaranteed that leftSide <= rightSide. */
	public final int rightSide;
	
	
	public final int width;
	
	/**
	 * Simple constructor that takes two points on the number line.
	 * @param _a
	 * @param _b
	 */
	public Interval(int _a, int _b){
		leftSide = Math.min(_a, _b);
		rightSide = Math.max(_a, _b);
		width = rightSide - leftSide;
	}
	
	/** Tests to see if the value is contained on the interval. */
	public boolean containsValue(Number _val) {return (leftSide<=_val.doubleValue() && _val.doubleValue()<=rightSide);}
	
	/** Returns the number of integers contained in the interval */
	public int getNumValues() {return rightSide-leftSide+1;}
	
	/** Attempts to map the value to the interval, 'squeezing' it if necessary. */
	public int squeeze(int _val) {
		if(_val > rightSide)
			return rightSide;
		else if(_val < leftSide)
			return leftSide;
		else
			return _val;
	}
	
	/** Overridden equals method */
	public boolean equals(Object _obj){
		if(_obj == null) return false;
		else if(!(_obj instanceof Interval)) return false;
		else{
			Interval i = Interval.class.cast(_obj);
			return (i.leftSide == leftSide && i.rightSide == rightSide);
		}
	}
	
	/** Overridden toString method */
	public String toString(){
		String ret = "[Interval=";
		ret += "[leftSize="+leftSide+"]";
		ret += "[rightSize="+rightSide+"]";
		ret += "[width="+width+"]";
		ret += "]";
		return ret;
	}
	
	
};