package org.helioviewer.jhv.gui;

import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.net.URL;

import javax.swing.ImageIcon;

/**
 * This class provides access to all images, icons and cursors which are used by
 * the program.
 * 
 * @author caplins
 * @author dmueller
 * @author Stephan Pagel
 */
public class IconBank {
	
	/**
	 * The enum has all the icons, you supply these enums to the getIcon method.
	 * @author caplins
	 * cleaned up unused icons
	 * @author dmueller
	 * 
	 */
	public static enum JHVIcon{
		PROPERTIES("properties16.gif"),
		BLANK("blank_square.gif"),
		// the icons currently used for back/forward should be replaced
		PLAY("play_dm.png"),
		PAUSE("agt_pause-queue.png"),
		BACK("agt_back.png"),
		FORWARD("agt_forward.png"),
		ZOOM_IN("zoomIn24.png"),
		ZOOM_FIT("zoomFit24.png"),
		ZOOM_OUT("zoomOut24.png"),
		ZOOM_1TO1("zoom1to124.png"),
		LEFT("arrow_left.gif"),
		RIGHT("arrow_right.gif"),
		UP("arrow_up.gif"),
		DOWN("arrow_down.gif"),
		RIGHT2("arrow.plain.right.gif"),
		DOWN2("arrow.plain.down.gif"),
		ADD("new_window.png"),
		DOWNLOAD("download_dm.png"),
		OPEN_HAND("OpenedHand.gif"),
		OPEN_HAND_SMALL("OpenedHand2.gif"),	
		CLOSED_HAND("ClosedHand.gif"),
		PAN("pan.png"),
		SELECT("select.png"),
		FOCUS("crosshairs.png"), 
		// need better icon for "invert":
		//INVERT("invert.gif"),
		VISIBLE("layer_visible_dm.png"),
		HIDDEN("layer_invisible_dm.png"),
		REMOVE_LAYER("button_cancel.png"),
		INFO("info.png"),
		CHECK("button_ok.png"),
		EX("button_cancel.png"),
		RUBBERBAND("rubberband.gif"),
		NOIMAGE("NoImageLoaded_256x256.png"),
		CONNECTED("connected_dm.png"),
		DISCONNECTED("not_connected_dm.png"),
		MOVIE_LINK("unlocked.png"),
		MOVIE_UNLINK("locked.png"),
		SPLASH("jhv_splash.png"),
		HVLOGO_SMALL("hvImage_64x64.png"),
		SIMPLE_ARROW_RIGHT("Arrow-Right.png"),
		SIMPLE_ARROW_LEFT("Arrow-Left.png"),
		SIMPLE_DOUBLEARROW_RIGHT("DoubleArrow-Right.png"),
		SIMPLE_DOUBLEARROW_LEFT("DoubleArrow-Left.png"),
		DATE("date.png"),
		SHOW_MORE("1downarrow1.png"),
		INVERT("invert.png"),
		LOADING("Loading_256x256.png");

				
		private final String fname;
		JHVIcon(String _fname) {fname = _fname;} 
		String getFilename() {return fname;}
	};
	
	/** The location of the image files relative to this folder. */
	private static final String RESOURCE_PATH = "resources/images/";
	
    /**
     * Returns the ImageIcon associated with the given enum
     * 
     * @param _icon
     *            enum which represents the image
     * @return the image icon of the given enum
     * */
	public static ImageIcon getIcon(JHVIcon _icon){
		URL imgURL = IconBank.class.getResource(RESOURCE_PATH+_icon.getFilename());
		return new ImageIcon(imgURL);
	}
	
    /**
     * Returns the Image with the given enum.
     * 
     * @param icon
     *            Name of the image which should be loaded
     * @return Image for the given name or null if it fails to load the image.
     * */
	public static BufferedImage getImage(JHVIcon icon) {
	    
	    ImageIcon imageIcon = getIcon(icon);
	    
	    if (imageIcon == null)
	        return null;
	    
	    Image image = imageIcon.getImage();	   
	    
	    if (image != null && image.getWidth(null) > 0 && image.getHeight(null) > 0) {
	        BufferedImage bi = new BufferedImage(image.getWidth(null), image.getHeight(null), BufferedImage.TYPE_INT_ARGB);
	        
	        Graphics g = bi.getGraphics();
	        g.drawImage(image, 0, 0, null);
	        g.dispose();
	        
	        return bi;
	    } 
	    
	    return null;
	}
	
};
