package org.helioviewer.jhv.gui.actions;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;

import javax.swing.AbstractAction;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;

import org.helioviewer.jhv.JHVDirectory;
import org.helioviewer.jhv.gui.ImageViewerGui;

/**
 * Action to save a screenshot in desired image format at
 * desired location.
 * 
 * <p>Therefore, opens a save dialog to choose format, name
 * and location.
 * 
 * @author Markus Langenberg
 */
public class SaveScreenshotAsAction extends AbstractAction {
	
	private static final long serialVersionUID = 1L;
	
	/**
	 * Default constructor.
	 */
	public SaveScreenshotAsAction() {
		super("Save Screenshot As...");
		putValue(SHORT_DESCRIPTION, "Save Screenshot to Chosen Folder");
	}
	
	/**
	 * {@inheritDoc}
	 */
	public void actionPerformed(ActionEvent e) {
		final JFileChooser fileChooser = new JFileChooser();
		fileChooser.setAcceptAllFileFilterUsed(false);
		fileChooser.addChoosableFileFilter(new JPGFilter());
		fileChooser.addChoosableFileFilter(new PNGFilter());
		
		fileChooser.setSelectedFile(new File(JHVDirectory.EXPORTS.getPath() +  SaveScreenshotAction.getDefaultFileName()));
		int retVal = fileChooser.showSaveDialog(ImageViewerGui.getMainFrame());
		
		if(retVal == JFileChooser.APPROVE_OPTION) {
			File selectedFile = fileChooser.getSelectedFile();
			
			if(!fileChooser.getFileFilter().accept(selectedFile)) {
				String filename = selectedFile.getPath();
				if(!filename.endsWith(fileChooser.getFileFilter().getDescription())) {
					filename += fileChooser.getFileFilter().getDescription();
				}
				selectedFile = new File(filename);
			}
			
			try {
				ImageViewerGui.getSingletonInstance().getMainView().saveScreenshot(
						fileChooser.getFileFilter().getDescription().substring(1), selectedFile);
			} catch (IOException e1) {
				e1.printStackTrace();
			}
		}
	}
	
	private class PNGFilter extends FileFilter {

		/**
		 * {@inheritDoc}
		 */
		public boolean accept(File f) {
			return f.getName().toLowerCase().endsWith(".png");
		}

		public String getDescription() {
			return ".png";
		}
		
	}
	
	private class JPGFilter extends FileFilter {

		/**
		 * {@inheritDoc}
		 */
		public boolean accept(File f) {
			String name = f.getName().toLowerCase();
			return name.endsWith(".jpg") || name.endsWith(".jpeg");
		}

		public String getDescription() {
			return ".jpg";
		}
		
	}
}
