package org.helioviewer.jhv.gui.components;

import java.awt.Dimension;

import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.helioviewer.viewmodel.view.View;
import org.helioviewer.viewmodel.view.ViewListener;
import org.helioviewer.viewmodel.view.jp2view.JP2View;
import org.helioviewer.viewmodelplugin.filter.FilterPanel;

/**
 * A class that is used to display a slider that updates the quality layers used
 * for an image.
 */
public class QualitySlider extends JPanel implements ChangeListener, ViewListener {

	private static final long serialVersionUID = 1L;
	private JSlider slider;
    private JLabel qualityLabel;
    private JP2View jp2View;
    
    /** The public constructor that sets up the panel containing the slider */
    public QualitySlider(JP2View view) {
    	if(view == null) {
    		System.err.println("QualitySlider without view");
    		return;
    	}
    	
    	setLayout(new BoxLayout(this, BoxLayout.LINE_AXIS));

        jp2View = view;
        
        JLabel title = new JLabel("Quality:");
        title.setPreferredSize(new Dimension(FilterPanel.titleWidth, FilterPanel.height));
        add(title);

        // Create the slider.
        slider = new JSlider(JSlider.HORIZONTAL);
        slider.setMaximum(jp2View.getMaximumNumQualityLayers());
        slider.setMinimum(1);
        slider.setValue(jp2View.getCurrentNumQualityLayers());
        slider.setEnabled(true);
        slider.setMajorTickSpacing(1);
        slider.setPaintTicks(true);
        slider.setSnapToTicks(true);
        slider.setPreferredSize(new Dimension(150, slider.getPreferredSize().height));
        slider.addChangeListener(this);
        add(slider);
        
        qualityLabel = new JLabel(slider.getValue() + "/" + slider.getMaximum());
        qualityLabel.setHorizontalAlignment(JLabel.RIGHT);
        qualityLabel.setPreferredSize(new Dimension(FilterPanel.valueWidth, FilterPanel.height));
        add(qualityLabel);  
        
        jp2View.addViewListener(this);
        
    }

    /** A method to restore the default setting */
    public void setToDefault() {
        if (jp2View != null) {
            slider.setValue(jp2View.getMaximumNumQualityLayers());
        }
    }

    public void stateChanged(ChangeEvent e) {
        int quality = slider.getValue();

        jp2View.setNumQualityLayers(quality);
        qualityLabel.setText(slider.getValue() + "/" + slider.getMaximum());
    }

	public void viewChanged(View sender, org.helioviewer.viewmodel.changeevent.ChangeEvent aEvent) {
		slider.setValue(jp2View.getCurrentNumQualityLayers());
	} 
}