package org.helioviewer.jhv.gui.components;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.EtchedBorder;

import org.helioviewer.jhv.gui.interfaces.StatusPanelPlugin;

/**
 * A class used for displaying information and statuses in a panel at the very bottom of JHV.
 * @author caplins
 * @author Benjamin Wamsler
 * @author Desmond Amadigwe
 * @author Alen Agheksanterian
 * @author Markus Langenberg
 */
public class StatusPanel extends JPanel {
	
	// ////////////////////////////////////////////////////////////////
	// Definitions
	// ////////////////////////////////////////////////////////////////
	
	private static final long serialVersionUID = 1L;
	private static final int HEIGHT = 30;

    private JPanel leftPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
    private JPanel rightPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));

    public static JLabel statusInfoLabel = new JLabel(""); 
    
    public enum Alignment {LEFT, RIGHT};
    
    // ////////////////////////////////////////////////////////////////
	// Methods
	// ////////////////////////////////////////////////////////////////

	/** The public constructor that initializes the panel. */
	public StatusPanel(int leftMargin, int rightMargin) {
		super(new BorderLayout());
		
		add(leftPanel, BorderLayout.WEST);
		add(rightPanel, BorderLayout.EAST);
		
		if(leftMargin != 0) {			
			statusInfoLabel.setPreferredSize(new Dimension(leftMargin, 20));
	        leftPanel.add(statusInfoLabel);
		}
		
		if(rightMargin != 0) {
			JPanel spacer = new JPanel();
			spacer.setPreferredSize(new Dimension(rightMargin, 1));
			rightPanel.add(spacer);
		}
        
		setPreferredSize(new Dimension(0, HEIGHT));
		setBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED));
	}

    public void addPlugin(StatusPanelPlugin newPlugin, Alignment alignment) {
        if (newPlugin == null || !(newPlugin instanceof JLabel)) {
            return;
        }

        if(alignment == Alignment.LEFT) {
        	leftPanel.add((JLabel)newPlugin);
        } else {
        	rightPanel.add((JLabel)newPlugin, 0);
        }
    }

    public void removePlugin(StatusPanelPlugin oldPlugin) {
        if(oldPlugin == null || !(oldPlugin instanceof JLabel)) {
            return;
        }

        leftPanel.remove((JLabel)oldPlugin);
        rightPanel.remove((JLabel)oldPlugin);
    }

    public static void setStatusInfoText(String text) {
    	statusInfoLabel.setText(text);
    } 
}
