package org.helioviewer.jhv.gui.components;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JPanel;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;

import org.helioviewer.jhv.gui.actions.SetFocusSelectionAction;
import org.helioviewer.jhv.gui.actions.SetPanSelectionAction;
import org.helioviewer.jhv.gui.actions.SetZoomBoxSelectionAction;
import org.helioviewer.jhv.gui.actions.Zoom1to1Action;
import org.helioviewer.jhv.gui.actions.ZoomFitAction;
import org.helioviewer.jhv.gui.actions.ZoomInAction;
import org.helioviewer.jhv.gui.actions.ZoomOutAction;

public class TopToolBar extends JPanel {

    private static final long serialVersionUID = 1L;

    public enum SelectionMode {PAN, ZOOMBOX, FOCUS};

    private JToggleButton panButton;
    private JToggleButton zoomBoxButton;
    private JToggleButton focusButton;

    public TopToolBar() {
        setLayout(new BoxLayout(this, BoxLayout.LINE_AXIS));

        // Zoom
        JToolBar zoomToolBar = new JToolBar("Zoom");
        zoomToolBar.setRollover(true);

        final int b = 4;
        zoomToolBar.add(new ZoomInAction()).setBorder(BorderFactory.createEmptyBorder(b,b,b,b));
        zoomToolBar.add(new ZoomOutAction()).setBorder(BorderFactory.createEmptyBorder(b,b,b,b));
        zoomToolBar.add(new ZoomFitAction()).setBorder(BorderFactory.createEmptyBorder(b,b,b,b));
        zoomToolBar.add(new Zoom1to1Action()).setBorder(BorderFactory.createEmptyBorder(b,b,b,b));
        add(zoomToolBar);

        // Selection
        JToolBar selectionToolBar = new JToolBar("SelectionMode");
        ButtonGroup group = new ButtonGroup();

        panButton = new JToggleButton(new SetPanSelectionAction());
        panButton.setSelected(true);
        panButton.setText("");
        selectionToolBar.add(panButton);
        group.add(panButton);

        zoomBoxButton = new JToggleButton(new SetZoomBoxSelectionAction());
        zoomBoxButton.setText("");
        selectionToolBar.add(zoomBoxButton);
        group.add(zoomBoxButton);

        focusButton = new JToggleButton(new SetFocusSelectionAction());
        focusButton.setText("");
        selectionToolBar.add(focusButton);
        group.add(focusButton);

        add(selectionToolBar);

    }

    public void setActiveSelectionMode(SelectionMode mode) {
        switch (mode) {
        case PAN:     panButton.doClick();     break;
        case ZOOMBOX: zoomBoxButton.doClick(); break;
        case FOCUS:   focusButton.doClick();   break;
        }
    }

}
