package org.helioviewer.jhv.gui.components.statusplugins;

import java.awt.Dimension;

import javax.swing.BorderFactory;
import javax.swing.JLabel;

import org.helioviewer.jhv.gui.ViewListenerDistributor;
import org.helioviewer.jhv.gui.interfaces.StatusPanelPlugin;
import org.helioviewer.viewmodel.changeevent.ChangeEvent;
import org.helioviewer.viewmodel.changeevent.SubImageDataChangedReason;
import org.helioviewer.viewmodel.view.MovieView;
import org.helioviewer.viewmodel.view.View;
import org.helioviewer.viewmodel.view.ViewHelper;
import org.helioviewer.viewmodel.view.ViewListener;

/** Status panel for displaying the framerate for image series.
 * 
 * <p>The information of this panel is always shown for the
 * active layer.
 * 
 * <p>This panel is not visible, if the active layer is not
 * an image series.
 * 
 * @author Markus Langenberg
 */
public class FramerateStatusPanel extends JLabel implements ViewListener,
        StatusPanelPlugin {

	private static final long serialVersionUID = 1L;
	private MovieView view;

	/**
	 * Default constructor.
	 */
	public FramerateStatusPanel() {
        setBorder(BorderFactory.createEtchedBorder());

        setPreferredSize(new Dimension(70, 20));
        setText("fps:");
        
        ViewListenerDistributor.getSingletonInstance().addViewListener(this);
    }

    /** 
     * Updates the shown frames per second.
     */
    private void updateFps() {

        if (view != null) {
        	setVisible(true);
        	
        	String fps;

        	if(view.isMoviePlaying())  {
	        	fps = Float.toString(view.getActualFramerate());
	        	fps = fps.substring(0, Math.max(fps.lastIndexOf('.'), fps.lastIndexOf(',')) + 2);
        	} else {
        		fps = "---";
        	}
        	
            setText("fps: " + fps);
        } else {
        	setVisible(false);
        }
    }

    /**
     * {@inheritDoc}
     */
    public void viewChanged(View sender, ChangeEvent aEvent) {
    	if(aEvent.reasonOccurred(ActiveLayerChangedReason.class)) {
    		view = ViewHelper.getViewAdapter(sender, MovieView.class);
    		updateFps();
    	}
    	
    	if(aEvent.reasonOccurred(SubImageDataChangedReason.class)) {
    		updateFps();
    	}
    }
}
