package org.helioviewer.jhv.gui.components.statusplugins;

import java.awt.Dimension;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JLabel;

import org.helioviewer.jhv.gui.IconBank;
import org.helioviewer.jhv.gui.ViewListenerDistributor;
import org.helioviewer.jhv.gui.IconBank.JHVIcon;
import org.helioviewer.jhv.gui.interfaces.StatusPanelPlugin;
import org.helioviewer.viewmodel.metadata.HelioviewerMetaData;
import org.helioviewer.viewmodel.metadata.MetaData;
import org.helioviewer.viewmodel.changeevent.ChangeEvent;
import org.helioviewer.viewmodel.view.MetaDataView;
import org.helioviewer.viewmodel.view.View;
import org.helioviewer.viewmodel.view.ViewListener;

/** Status panel for displaying whether the image contains
 * helioviewer meta data or not.
 * 
 * <p>The information of this panel is always shown for the
 * active layer.
 * 
 * <p>If there is no layer present, this panel will be invisible.
 * 
 * @author Markus Langenberg
 */
public class MetaDataStatusPanel extends JLabel implements ViewListener,
        StatusPanelPlugin {

	private static final long serialVersionUID = 1L;
	private View activeView;
	
	private static final Icon checkIcon = IconBank.getIcon(JHVIcon.CHECK);
	private static final Icon exIcon = IconBank.getIcon(JHVIcon.EX);

	/**
	 * Default constructor.
	 */
    public MetaDataStatusPanel() {
        setBorder(BorderFactory.createEtchedBorder());

        setPreferredSize(new Dimension(60, 20));
        setText("Meta:");
        setIcon(exIcon);
        setVerticalTextPosition(JLabel.CENTER);
        setHorizontalTextPosition(JLabel.LEFT);
        
        ViewListenerDistributor.getSingletonInstance().addViewListener(this);
    }

    /**
     * Updates the meta data status
     */
    public void updateXMLStatus() {
        if (activeView != null) {
        	setVisible(true);
        	
            MetaDataView metaDataView = activeView.getAdapter(
                            MetaDataView.class);

            if (metaDataView != null) {
                MetaData m = metaDataView.getMetaData();
                if (m instanceof HelioviewerMetaData)
                    setIcon(checkIcon);
                else
                    setIcon(exIcon);
            }
        } else {
        	setVisible(false);
        }

    }

    /**
     * {@inheritDoc}
     */
    public void viewChanged(View sender, ChangeEvent aEvent) {
    	if(aEvent.reasonOccurred(ActiveLayerChangedReason.class)) {
    		activeView = sender;
    		updateXMLStatus();
    	}
    }
}
