package org.helioviewer.jhv.gui.components.statusplugins;

import java.awt.Dimension;

import javax.swing.BorderFactory;
import javax.swing.JLabel;

import org.helioviewer.jhv.gui.ViewListenerDistributor;
import org.helioviewer.jhv.gui.interfaces.StatusPanelPlugin;
import org.helioviewer.viewmodel.changeevent.ChangeEvent;
import org.helioviewer.viewmodel.view.View;
import org.helioviewer.viewmodel.view.ViewHelper;
import org.helioviewer.viewmodel.view.ViewListener;
import org.helioviewer.viewmodel.view.jp2view.JP2View;

/** Status panel for displaying quality layer currently used.
 * 
 * <p>The information of this panel is always shown for the
 * active layer.
 * 
 * <p>This panel is not visible, if the active layer is not
 * an JPG2000 image.
 * 
 * @author Markus Langenberg
 */
public class QualityStatusPanel extends JLabel implements ViewListener,
        StatusPanelPlugin {

	private static final long serialVersionUID = 1L;
	private JP2View jp2View;

	/**
	 * Default constructor.
	 */
    public QualityStatusPanel() {
        setPreferredSize(null);
        setText("");
        setBorder(BorderFactory.createEtchedBorder());

        setPreferredSize(new Dimension(100, 20));
        setText("Quality: ");
        
        ViewListenerDistributor.getSingletonInstance().addViewListener(this);
    }

    /** 
     * Updates the quality layer currently used.
     */
    public void updateQualityLayers() {
    	
    	if (jp2View != null) {
            int qlayers = jp2View.getCurrentNumQualityLayers();
            int maxQlayers = jp2View.getMaximumNumQualityLayers();
            setVisible(true);
            setText("Quality: " + qlayers + "/" + maxQlayers);
        } else {
        	setVisible(false);
        }
    }

    /**
     * {@inheritDoc}
     */
    public void viewChanged(View sender, ChangeEvent aEvent) {
    	if(aEvent.reasonOccurred(ActiveLayerChangedReason.class)) {
    		jp2View = ViewHelper.getViewAdapter(sender, JP2View.class);
    	}
    	
    	updateQualityLayers();
    }

}
