package org.helioviewer.jhv.gui.components.statusplugins;

import java.awt.Dimension;

import javax.swing.BorderFactory;
import javax.swing.JLabel;

import org.helioviewer.base.math.Vector2dDouble;
import org.helioviewer.jhv.gui.ViewListenerDistributor;
import org.helioviewer.jhv.gui.interfaces.StatusPanelPlugin;
import org.helioviewer.viewmodel.metadata.ImageSizeMetaData;
import org.helioviewer.viewmodel.changeevent.ChangeEvent;
import org.helioviewer.viewmodel.changeevent.MetaDataChangedReason;
import org.helioviewer.viewmodel.changeevent.RegionChangedReason;
import org.helioviewer.viewmodel.changeevent.ViewportChangedReason;
import org.helioviewer.viewmodel.region.Region;
import org.helioviewer.viewmodel.region.StaticRegion;
import org.helioviewer.viewmodel.view.MetaDataView;
import org.helioviewer.viewmodel.view.RegionView;
import org.helioviewer.viewmodel.view.View;
import org.helioviewer.viewmodel.view.ViewHelper;
import org.helioviewer.viewmodel.view.ViewListener;
import org.helioviewer.viewmodel.view.ViewportView;
import org.helioviewer.viewmodel.viewport.StaticViewport;
import org.helioviewer.viewmodel.viewport.Viewport;

/** Status panel for displaying the current zoom.
 * 
 * <p>A displayed zoom of 100% means that one pixel one the
 * screen corresponds to exactly one pixel in the native resolution
 * of the image.
 * 
 * <p>The information of this panel is always shown for the
 * active layer.
 * 
 * <p>If there is no layer present, this panel will be invisible.
 */
public class ZoomStatusPanel extends JLabel implements ViewListener,
        StatusPanelPlugin {

	private static final long serialVersionUID = 1L;
	private View view;
	
	private Region lastRegion = StaticRegion.createAdaptedRegion(0, 0, 0, 0);
	private Viewport lastViewport = StaticViewport.createAdaptedViewport(0, 0);
	private double lastUnitsPerPixel = 0;

	/**
	 * Default constructor.
	 */
	public ZoomStatusPanel() {
        setBorder(BorderFactory.createEtchedBorder());

        setPreferredSize(new Dimension(100, 20));
        setText("Zoom:");
        
        ViewListenerDistributor.getSingletonInstance().addViewListener(this);
    }

    /** 
     * Updates the displayed zoom.
     */
    private void updateZoomLevel() {

        if (view != null) {
        	Region region = view.getAdapter(RegionView.class).getRegion();
            Viewport viewport = view.getAdapter(ViewportView.class).getViewport();
            double unitsPerPixel = ((ImageSizeMetaData) view.getAdapter(
		                    MetaDataView.class).getMetaData()).getUnitsPerPixel();
            
            if(lastUnitsPerPixel != unitsPerPixel
            		|| region.getSize() != lastRegion.getSize()
            		|| viewport.getSize() != lastViewport.getSize()) {
	
	            long zoom = Math.round(new Vector2dDouble(
	            		ViewHelper.calculateViewportImageSize(viewport, region).getSizeVector()).scale(unitsPerPixel)
	                    .invertedScale(region.getSize()).scale(100.00).getX());
	            
	            lastRegion = region;
	            lastViewport = viewport;
	            lastUnitsPerPixel = unitsPerPixel;
	            
	            setText("Zoom: " + zoom + "%");
	            setVisible(true);
            }
        } else {
        	setVisible(false);
        }
    }

    public void viewChanged(View sender, ChangeEvent aEvent) {
    	if(aEvent.reasonOccurred(ActiveLayerChangedReason.class)) {
    		view = sender;
    		updateZoomLevel();
    	}
    	
    	if(aEvent.reasonOccurred(MetaDataChangedReason.class)
    	|| aEvent.reasonOccurred(RegionChangedReason.class)
    	|| aEvent.reasonOccurred(ViewportChangedReason.class)) {
    		updateZoomLevel();
    	}
    }

}
