package org.helioviewer.jhv.gui.dialogs;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;

import org.helioviewer.jhv.JHVGlobals;
import org.helioviewer.jhv.gui.IconBank;
import org.helioviewer.jhv.gui.ImageViewerGui;
import org.helioviewer.jhv.gui.IconBank.JHVIcon;
import org.helioviewer.jhv.gui.interfaces.ShowableDialog;

/** Dialog that is used to display information about the program.
 * 
 * <p>This includes version and contact informations.
 */
public final class AboutDialog extends JDialog implements ActionListener, ShowableDialog, HyperlinkListener {

	private static final long serialVersionUID = 1L;
	
	private final JButton closeButton = new JButton("Close");
    
    /** 
	 * Default constructor. 
	 */
	public AboutDialog() {
        super(ImageViewerGui.getMainFrame(), "About JHelioviewer", true);
    	setLayout(new BorderLayout());
    	setResizable(false);
    	        
        JLabel logo = new JLabel(IconBank.getIcon(JHVIcon.HVLOGO_SMALL));
        logo.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
        add(logo, BorderLayout.WEST);

        Font font = this.getFont();
                
        JEditorPane content = new JEditorPane("text/html",
        		"<html><center><font style=\"font-family: '" + font.getFamily() + 
        		"'; font-size: " + font.getSize() + ";\">" + 
        		"<b>JHelioviewer 2.0.2 - Revision 105</b><br>" + 
        		'\u00A9' + "2010 ESA Helioviewer Team<br><br>" +
        		"JHelioviewer is released under the <br>" +
        		"<a href=JHelioviewer.txt>Mozilla Public License Version 1.1</a><br><br>" +
        		"<a href='http://www.jhelioviewer.org'>www.jhelioviewer.org</a><br><br>" +
        		"Contact: <a href='mailto:Daniel.Mueller@esa.int'>Daniel.Mueller@esa.int</a>" + 
        		"</font></center></html>"
        );
        content.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
        content.setEditable(false);
        content.setOpaque(false);
        content.addHyperlinkListener(this);
        add(content, BorderLayout.CENTER);
        
        JPanel boxPanel = new JPanel();
        boxPanel.setLayout(new BoxLayout(boxPanel, BoxLayout.PAGE_AXIS));
        
        boxPanel.add(new JSeparator());
        
        JEditorPane license = new JEditorPane("text/html",
        		"<html><font style=\"font-family: '" + font.getFamily() + 
        		"'; font-size: " + font.getSize() + ";\">" + 
        		
        		"This software uses the <a href=http://www.kakadusoftware.com>Kakadu JPEG2000 Toolkit</a>,<br> " + 
        		'\u00A9' + " 2009, NewSouth Innovations Ltd (NSI), <a href=Kakadu.txt>(License)</a><br>" +
        		
        		"<p>This software uses the <a href=http://kenai.com/projects/jogl>Java OpenGL Bindings (JOGL)</a>,<br>" +
        		'\u00A9' + " 2003-2007 Sun Microsystems, Inc., <a href=JOGL.txt>(License)</a><br>" +
        		
        		"<p>This software uses the <a href=https://gluegen.dev.java.net/>GlueGen Toolkit</a>,<br>" +
        		'\u00A9' + " 2003-2005 Sun Microsystems, Inc., <a href=GlueGen.txt>(License)</a><br>" +
        		
        		"<p>This software uses the <a href=http://developer.nvidia.com/object/cg_toolkit.html>Cg Compiler</a>,<br>" +
        		'\u00A9' + " 2009, NVIDIA Corp., <a href=Cg.txt>(License)</a><br>" + 
        		
        		"<p>This software uses the <a href=http://www.raben.com/articles/JulianDay/JulianDayTutorial.html>JulianDay class</a>,<br>" +
                '\u00A9' + " 2002, Raben Systems, Inc., <a href=JulianDay.txt>(License)</a><br>" + 
        		
        		"<p>This software uses the <a href=http://www.ffmpeg.org>FFmpeg project</a>.<br>" +
        		"It is licensed under the <a href=FFmpeg.txt>LGPLv2.1<a>.<br>" +
        		"Its source code can be downloaded <a href=http://jhelioviewer.org/libjhv/ffmpeg/source/ffmpeg-source.tar.bz2>here</a>.<br>"
        );
        
        license.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
        license.setEditable(false);
        license.setOpaque(false);
        license.addHyperlinkListener(this);
        boxPanel.add(license);
        
        JPanel closeButtonContainer = new JPanel(new FlowLayout(FlowLayout.RIGHT)); 
        closeButton.addActionListener(this);
        closeButtonContainer.add(closeButton);
        boxPanel.add(closeButtonContainer);
        
        add(boxPanel, BorderLayout.SOUTH);
    }

	/** 
	 * {@inheritDoc}
	 */
	public void showDialog() {
        pack();
        setSize(getPreferredSize());
        setLocationRelativeTo(ImageViewerGui.getMainFrame());
        setVisible(true);
    }

    /**
     * Closes the dialog.
     */
    public void actionPerformed(ActionEvent a) {
        if (a.getSource() == this.closeButton) {
            this.dispose();
        }
    }

    /**
     * Opens a browser or email client after clicking on a hyperlink.
     */
	public void hyperlinkUpdate(HyperlinkEvent e) {
		if(e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
			if(e.getURL() == null) {
				TextDialog textDialog = new TextDialog("License - " + e.getDescription().substring(0, e.getDescription().indexOf('.')),
						ImageViewerGui.class.getResource("resources/licenses/" + e.getDescription()));
				textDialog.showDialog();
			} else {
				JHVGlobals.openURL(e.getURL().toString());
			}
		}
	}
}
