package org.helioviewer.jhv.gui.dialogs;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.ComponentOrientation;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import javax.imageio.ImageIO;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.text.NumberFormatter;

import org.helioviewer.base.math.RectangleDouble;
import org.helioviewer.base.math.Vector2dDouble;
import org.helioviewer.jhv.JHVDirectory;
import org.helioviewer.jhv.JHVGlobals;
import org.helioviewer.jhv.ffmpeg.FFmpegLoader;
import org.helioviewer.jhv.gui.ImageViewerGui;
import org.helioviewer.jhv.gui.ViewchainFactory;
import org.helioviewer.jhv.gui.components.OverviewImagePanel;
import org.helioviewer.jhv.gui.controller.MainImagePanelMouseFocusController;
import org.helioviewer.jhv.gui.controller.OverviewImagePanelMousePanController;
import org.helioviewer.jhv.gui.interfaces.ShowableDialog;
import org.helioviewer.viewmodel.changeevent.ChangeEvent;
import org.helioviewer.viewmodel.changeevent.MetaDataChangedReason;
import org.helioviewer.viewmodel.changeevent.RegionChangedReason;
import org.helioviewer.viewmodel.changeevent.SubImageDataChangedReason;
import org.helioviewer.viewmodel.changeevent.ViewportChangedReason;
import org.helioviewer.viewmodel.imagedata.JavaBufferedImageData;
import org.helioviewer.viewmodel.metadata.MetaData;
import org.helioviewer.viewmodel.metadata.ObserverMetaData;
import org.helioviewer.viewmodel.region.Region;
import org.helioviewer.viewmodel.region.StaticRegion;
import org.helioviewer.viewmodel.view.AbstractView;
import org.helioviewer.viewmodel.view.ComponentView;
import org.helioviewer.viewmodel.view.LayeredView;
import org.helioviewer.viewmodel.view.LinkedMovieManager;
import org.helioviewer.viewmodel.view.MetaDataView;
import org.helioviewer.viewmodel.view.MovieView;
import org.helioviewer.viewmodel.view.RegionView;
import org.helioviewer.viewmodel.view.SubimageDataView;
import org.helioviewer.viewmodel.view.SynchronizeOverviewChainView;
import org.helioviewer.viewmodel.view.TimedMovieView;
import org.helioviewer.viewmodel.view.View;
import org.helioviewer.viewmodel.view.ViewHelper;
import org.helioviewer.viewmodel.view.ViewListener;
import org.helioviewer.viewmodel.view.ViewportView;
import org.helioviewer.viewmodel.view.jp2view.JHVJP2View;
import org.helioviewer.viewmodel.view.jp2view.JHVJP2View.ReaderMode;
import org.helioviewer.viewmodel.view.jp2view.datetime.ImmutableDateTime;
import org.helioviewer.viewmodel.viewport.StaticViewport;
import org.helioviewer.viewmodel.viewport.Viewport;

/** Dialog o export movies to standard video formats.
 * 
 * <p>This class includes everything needed to export movies to
 * an external format. Therefore, it copies the existing view chain
 * and performs all its operations on this copy. To encode the
 * final result, the command line toll from FFmpeg is used.
 *  
 * @author Markus Langenberg
 */
public class ExportMovieDialog extends JDialog implements 
		ActionListener, PropertyChangeListener, ChangeListener, ViewListener, ShowableDialog {

	private static final long serialVersionUID = 1L;
		
	private JTextField txtTargetFile;
	private JButton cmdBrowseTargetFile;
	private JFormattedTextField txtWidth, txtHeight;
	private OverviewImagePanel imagePanel;
	private JSlider zoomSlider;
	private JSpinner speedSpinner;
	private JCheckBox loadFirstCheckBox;
	private JCheckBox useDifferentialRotationTracking;
	private JButton cmdExport, cmdCancel;
	private JProgressBar progressBar;
	
	private int linkedMovieManagerInstance = -1;
	private ViewDummy viewDummy;
	private SubimageDataView topmostView;
	private MovieView masterMovieView;
	private ViewListener additionalListener;
	private HashMap<TimedMovieView, StatusStruct> currentViewStatus = new HashMap<TimedMovieView, StatusStruct>();
	private Thread initThread, exportThread;
	private int currentFrame = 0;
	private boolean exportFinished = false;
	//private String lastExtensionUsed;
	private MovieFileFilter selectedOutputFormat = new MP4Filter();
	
	private class StatusStruct {
		public ImmutableDateTime currentDateTime = new ImmutableDateTime(0);
		public boolean regionHasUpdated = true;
	}

	/**
	 * Default constructor
	 */
	public ExportMovieDialog() {
		super(ImageViewerGui.getMainFrame(), "Export Movie", true);
		
		LayeredView layeredView = ImageViewerGui.getSingletonInstance().getMainView().getAdapter(LayeredView.class);
		for(int i=0; i<layeredView.getNumLayers(); i++) {
			MovieView movieView = layeredView.getLayer(i).getAdapter(MovieView.class);
			if(movieView != null) {
				movieView.pauseMovie();
			}
		}
		
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				exportFinished = true;
				finish();
			}
		});
		
		setLayout(new BorderLayout());
		setResizable(false);
		
		// Parameters
		JPanel parameterPanel = new JPanel(new GridBagLayout());
		parameterPanel.setComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(2, 2, 2, 2);
		
		// Target File
		c.gridy = 0;
		c.gridwidth = 1;
		cmdBrowseTargetFile = new JButton("Browse...");
		cmdBrowseTargetFile.addActionListener(this);
		parameterPanel.add(cmdBrowseTargetFile, c);
		c.gridwidth = 2;
		c.weightx = 1.0;
		
		String standardFile = new String(JHVDirectory.EXPORTS.getPath() + "JHV_movie_created_");
		
		SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd_HH.mm.ss");
		standardFile += dateFormat.format(new Date());
		standardFile += selectedOutputFormat.getExtension();
		
		txtTargetFile = new JTextField(standardFile);
		txtTargetFile.setPreferredSize(new Dimension(100, txtTargetFile.getPreferredSize().height));
		parameterPanel.add(txtTargetFile, c);
		c.gridwidth = 1;
		c.weightx = 0.0;
		parameterPanel.add(new JLabel("Export To:"), c);
				
		// Image panel
		c.gridy++;
		c.gridwidth = 2;
		c.gridheight = 8;
		JPanel imagePanelContainer = new JPanel(new FlowLayout(FlowLayout.CENTER, 2, 2));
		
		imagePanel = new OverviewImagePanel();
		imagePanel.setPreferredSize(new Dimension(210, 210));
		imagePanel.setInputController(new OverviewImagePanelMousePanController());
		imagePanelContainer.add(imagePanel);
		
		// Create view chain factory
		ViewchainFactory viewChainFactory = new ViewchainFactory(true);
		
		// Copy main view chain
		ComponentView componentView = 
			viewChainFactory.createViewchainMain(ImageViewerGui.getSingletonInstance().getMainView(), true);
		
		// Delete invisible layers
		LayeredView mainLayeredView = ImageViewerGui.getSingletonInstance().getMainView().getAdapter(LayeredView.class);
		LayeredView exportLayeredView = componentView.getAdapter(LayeredView.class);
		
		for(int i=mainLayeredView.getNumLayers()-1; i>=0; i--) {
			if(!mainLayeredView.isVisible(mainLayeredView.getLayer(i))) {
				exportLayeredView.removeLayer(i);
			}
		}
		
		imagePanel.setView(componentView);
		
		parameterPanel.add(imagePanelContainer, c);
		
		c.gridheight = 1;			
		
		// Width
		c.gridwidth = 1;
		txtWidth = new JFormattedTextField(new Integer(640));
		txtWidth.addPropertyChangeListener("value", this);
		((NumberFormat)((NumberFormatter) txtWidth.getFormatter()).getFormat()).setGroupingUsed(false);
		parameterPanel.add(txtWidth, c);
		parameterPanel.add(new JLabel("Width:"), c);
		
		// Height
		c.gridy++;
		txtHeight = new JFormattedTextField(new Integer(640));
		txtHeight.addPropertyChangeListener("value", this);
		((NumberFormat)((NumberFormatter) txtHeight.getFormatter()).getFormat()).setGroupingUsed(false);
		parameterPanel.add(txtHeight, c);
		parameterPanel.add(new JLabel("Height:"), c);
		
		// Zoom = Area used
		c.gridy++;
		zoomSlider = new JSlider(0, 100, 100);
		zoomSlider.setPreferredSize(new Dimension(100, 20));
		zoomSlider.addChangeListener(this);
		parameterPanel.add(zoomSlider, c);
		parameterPanel.add(new JLabel("Zoom:"), c);
		
		// Speed
		c.gridy++;
		speedSpinner = new JSpinner(new SpinnerNumberModel(20, 1, 99, 1));
		parameterPanel.add(speedSpinner, c);
		parameterPanel.add(new JLabel("Speed (fps):"), c);
		
		// Load First = Load all data via JPIP before exporting
		c.gridy++;
		c.gridwidth = 2;
		loadFirstCheckBox = new JCheckBox("Download full quality before export", true);
		parameterPanel.add(loadFirstCheckBox, c);
		
		// Use differential rotation tracking
		c.gridy++;
		useDifferentialRotationTracking = new JCheckBox("Use differential rotation tracking",
				ImageViewerGui.getSingletonInstance().getMainImagePanel().getInputController()
				instanceof MainImagePanelMouseFocusController);
		parameterPanel.add(useDifferentialRotationTracking, c);
		
		// Stretch
		c.gridy++;
		c.weighty = 1.0;
		parameterPanel.add(Box.createVerticalGlue(), c);
		
		// Buttons
		JPanel buttonPane = new JPanel();
		buttonPane.setLayout(new BoxLayout(buttonPane, BoxLayout.LINE_AXIS));

		cmdExport = new JButton("Export");
		cmdExport.addActionListener(this);
		
		cmdCancel = new JButton("Cancel");
		cmdCancel.addActionListener(this);
		
		buttonPane.add(Box.createHorizontalGlue());
		
		// Order depends on operation system
		if(System.getProperty("os.name").toUpperCase().contains("WIN")) {
			buttonPane.add(cmdExport);
			buttonPane.add(cmdCancel);
		} else {
			buttonPane.add(cmdCancel);
			buttonPane.add(cmdExport);
		}
		
		c.gridy++;
		parameterPanel.add(buttonPane, c);
		
		add(parameterPanel);
		
		// Progressbar
		progressBar = new JProgressBar(JProgressBar.HORIZONTAL, 0, 100);
		add(progressBar, BorderLayout.SOUTH);
		
		initThread = new Thread(new Runnable() {
			public void run() {
				initViewchain();
			}
		}, "InitMovieExport");
		initThread.start();
	}
	
	/**
	 * {@inheritDoc}
	 */
	public void actionPerformed(ActionEvent e) {
		// Start export
		if(e.getSource() == cmdExport) {
			if(exportThread != null)
				return;
			
			exportThread = new Thread(new Runnable() {
				public void run() {
					exportMovie();
				}
			}, "ExportMovie");
			exportThread.start();
			
		// Cancel export if running, close dialog in any case
		} else if(e.getSource() == cmdCancel) {
			exportFinished = true;
			finish();
			
		// Browse target file
		} else if(e.getSource() == cmdBrowseTargetFile) {
			
			// Open save-dialog
			final JFileChooser fileChooser = new JFileChooser();
			fileChooser.setAcceptAllFileFilterUsed(false);
			fileChooser.addChoosableFileFilter(new MOVFilter());
			fileChooser.addChoosableFileFilter(new MP4Filter());
			fileChooser.addChoosableFileFilter(new JPEGSetFilter());
			fileChooser.addChoosableFileFilter(new PNGSetFilter());
			fileChooser.addChoosableFileFilter(new JPEGArchiveFilter());
			fileChooser.addChoosableFileFilter(new PNGArchiveFilter());
			
			if(txtTargetFile.getText().toLowerCase().endsWith(selectedOutputFormat.getExtension())) {
				for(FileFilter fileFilter : fileChooser.getChoosableFileFilters()) {
					if(fileFilter.getClass() == selectedOutputFormat.getClass()) {
						fileChooser.setFileFilter(fileFilter);
						selectedOutputFormat = (MovieFileFilter) fileFilter;
					}
				}
			} else {
				for(FileFilter fileFilter : fileChooser.getChoosableFileFilters()) {
					if(txtTargetFile.getText().endsWith(((MovieFileFilter) fileFilter).getExtension())) {
						fileChooser.setFileFilter(fileFilter);
						selectedOutputFormat = (MovieFileFilter) fileFilter;
					}
				}
			}
			
			fileChooser.setSelectedFile(new File(txtTargetFile.getText()));
			int retVal = fileChooser.showDialog(ImageViewerGui.getMainFrame(), "OK");
			
			if(retVal == JFileChooser.APPROVE_OPTION) {
				String selectedFile = fileChooser.getSelectedFile().toString();
				
				MovieFileFilter newFilter = (MovieFileFilter) fileChooser.getFileFilter();
				
				// Remove last extension, if still present
				if(selectedFile.endsWith(selectedOutputFormat.getExtension())) {
					selectedFile = selectedFile.substring(0, selectedFile.lastIndexOf(selectedOutputFormat.getExtension()));
				}
				
				// Add new extension, if not present yet
				if(!selectedFile.endsWith(newFilter.getExtension())) {
					selectedFile += newFilter.getExtension();
				}
				txtTargetFile.setText(selectedFile);
				selectedOutputFormat = newFilter;
			}
		}
	}
	
	/**
	 * {@inheritDoc}
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		if(evt.getSource() == txtHeight || evt.getSource() == txtWidth) {
			viewDummy.setViewport(StaticViewport.createAdaptedViewport(
					(Integer) txtWidth.getValue(), (Integer) txtHeight.getValue()), new ChangeEvent());
			
			updateZoom();
			
		}
	}
	
	/**
	 * {@inheritDoc}
	 */
	public void stateChanged(javax.swing.event.ChangeEvent e) {
		double scaleFactor = zoomSlider.getValue() / 100.0;
		
		Vector2dDouble newSize = viewDummy.getMetaData().getPhysicalImageSize();
		double aspectRatio = viewDummy.getViewport().getAspectRatio();
		
		if(aspectRatio > 1) {
			newSize = newSize.scale(new Vector2dDouble(scaleFactor, scaleFactor / aspectRatio));
		} else {
			newSize = newSize.scale(new Vector2dDouble(scaleFactor * aspectRatio, scaleFactor));
		}
		
		Vector2dDouble offset = viewDummy.getRegion().getSize().subtract(newSize).scale(0.5);
		
		viewDummy.setRegion(StaticRegion.createAdaptedRegion(new RectangleDouble(
				viewDummy.getRegion().getLowerLeftCorner().add(offset),
				newSize).moveAndCropToOuterRectangle(viewDummy.getMetaData().getPhysicalRectangle())),
				new ChangeEvent());
	}
	
	/**
	 * {@inheritDoc}
	 */
	public synchronized void viewChanged(View sender, ChangeEvent aEvent) {
		
		// Get all meta data changes
		List<MetaDataChangedReason> metaDataChangedReasons = aEvent.getAllChangedReasonsByType(MetaDataChangedReason.class);
		
		// Update meta data list
		for(MetaDataChangedReason metaDataChangedReason : metaDataChangedReasons) {
			currentViewStatus.get((TimedMovieView) metaDataChangedReason.getView()).currentDateTime =  
					((ObserverMetaData) metaDataChangedReason.getNewMetaData()).getDateTime();
		}
		
		// Get all region changes
		List<RegionChangedReason> regionChangedReasons = aEvent.getAllChangedReasonsByType(RegionChangedReason.class);
		
		for(RegionChangedReason regionChangedReason : regionChangedReasons) {
			currentViewStatus.get((TimedMovieView) regionChangedReason.getView()).regionHasUpdated = true;
		}
		
		// If the view chain is not initialized yet or the export is already complete, return 
		if(topmostView == null || exportFinished
				|| topmostView.getSubimageData().getWidth() != viewDummy.getViewport().getWidth()
				|| topmostView.getSubimageData().getHeight() != viewDummy.getViewport().getHeight()) {
			return;
		}
		
		// Check, if all regions are up to date
		for(StatusStruct status : currentViewStatus.values()) {
			if(!status.regionHasUpdated) {
				return;
			}
		}
		
		// Compare all time stamps from the meta data (which are updated after decoding the image
		// with all time stamps from the movie view (which are updated immediately, thus are out of sync
		// a little). If the are not equal, the decoder is still running, thus the frame is not ready
		// for being exported.
		for(TimedMovieView view : currentViewStatus.keySet()) {
			if(view.getCurrentFrameDateTime().compareTo(currentViewStatus.get(view).currentDateTime) != 0) {
				return;
			}
		}
		
		// Get frame
		BufferedImage input = ((JavaBufferedImageData) topmostView.getSubimageData()).getBufferedImage();
		
		if(input == null) {
			return;
		}
		
		// Convert to RGB image
		BufferedImage output = new BufferedImage(input.getWidth(), input.getHeight(), BufferedImage.TYPE_3BYTE_BGR);
		
		// Fill background with black color (in case the frame has opaque areas)
		Graphics2D g = output.createGraphics();
		g.setColor(Color.BLACK);
		g.fillRect(0, 0, input.getWidth(), input.getHeight());
		g.drawImage(input, null, 0, 0);
		
		// Write the image to temporary file
		try {
			File targetFile;
			if(selectedOutputFormat.useTempDirectoryForIntermediateImages()) {
				String frameNumber = Integer.toString(currentFrame);
				while(frameNumber.length() < 4) {
					frameNumber = "0" + frameNumber;
				}
				targetFile = new File(JHVDirectory.TEMP.getPath() + "frame" + 
						frameNumber + selectedOutputFormat.getIntermediateExtension());
			} else {
				String frameNumber = Integer.toString(currentFrame);
				while(frameNumber.length() < 4) {
					frameNumber = "0" + frameNumber;
				}
				targetFile = new File(txtTargetFile.getText().replace(
						selectedOutputFormat.getIntermediateExtension(),
						"_" + frameNumber + selectedOutputFormat.getIntermediateExtension()));
			}
			ImageIO.write(output, selectedOutputFormat.getIntermediateExtension().replace(".", ""), targetFile);
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		currentFrame++;
		
		// If last frame is reached, merge temporary files to movie
		if(masterMovieView.getCurrentFrameNumber() == masterMovieView.getMaximumFrameNumber()) {
			
			// If intermediate and final format are the same, nothing is left to do
			if(selectedOutputFormat.getExtension() != selectedOutputFormat.getIntermediateExtension()) {
				
				// ZIP-File: Compress intermediate files
				if(selectedOutputFormat.getExtension().equalsIgnoreCase(".zip")) {
					try {
						ZipOutputStream out = new ZipOutputStream(
								new BufferedOutputStream(new FileOutputStream(txtTargetFile.getText())));
						out.setMethod(ZipOutputStream.DEFLATED);
						
						final int bufferSize = 2048;
						byte data[] = new byte[bufferSize];
						
						File[] inputFiles = JHVDirectory.TEMP.getFile().listFiles(new FilenameFilter() {
							public boolean accept(File dir, String name) {
								return (name.contains("frame")
										&& name.toLowerCase().endsWith(selectedOutputFormat.getIntermediateExtension()));
							}
						});

						for (File file : inputFiles) {
							BufferedInputStream fileStream = new BufferedInputStream(new FileInputStream(file), bufferSize);
							ZipEntry entry = new ZipEntry(file.getName());
							out.putNextEntry(entry);
							
							int count;
							while ((count = fileStream.read(data, 0, bufferSize)) != -1) {
								out.write(data, 0, count);
							}
							fileStream.close();
						}
						out.close();
					} catch (Exception e) {
						e.printStackTrace();
					}

				
				// MOVIE-File: Create Movie using FFmpeg
				} else {
			
					// Build command for calling FFmpeg
					String commandLine;
					if(FFmpegLoader.getffmpegPath().contains(" ")) {
						commandLine = "\"" + FFmpegLoader.getffmpegPath() + "\"";
					} else {
						commandLine = FFmpegLoader.getffmpegPath();
					}
					
					commandLine += " -s " + txtWidth.getValue() + "x" + txtHeight.getValue() + " -sameq -y -r " + speedSpinner.getValue() + " -i ";
					
					if(JHVDirectory.TEMP.getPath().contains(" ")) {
						commandLine += "\"" + JHVDirectory.TEMP.getPath() + "frame%04d.bmp\" ";
					} else {
						commandLine += JHVDirectory.TEMP.getPath() + "frame%04d.bmp ";
					}
					
					if(txtTargetFile.getText().contains(" ")) {
						commandLine += "\"" + txtTargetFile.getText() + "\"";
					} else {
						commandLine += txtTargetFile.getText();
					}
					
					// Run FFmpeg
					try {
						Runtime.getRuntime().exec(commandLine);
			        } catch (IOException e) {
			            e.printStackTrace();
			        }
				}
			}
	        
	        exportFinished = true;
	        
			// Close dialog. This has to happen in a different thread, because
	        // changing a view listener during viewChanged() causes an exception.
	        Timer timer = new Timer("CloseMovieExport");
			timer.schedule(new TimerTask() {
				public void run() {
					finish();
				}
			}, 200);
			return;
		}
		
		if(additionalListener != null) {
			TimedMovieView masterMovieView = LinkedMovieManager.getActiveInstance().getMasterMovie();
			additionalListener.viewChanged(topmostView, 
					new ChangeEvent(new MetaDataChangedReason(masterMovieView, ((MetaDataView) masterMovieView).getMetaData())));
			
			if(currentFrame > 2) {
				for(StatusStruct status : currentViewStatus.values()) {
					status.regionHasUpdated = false;
				}
			} else {
				additionalListener.viewChanged(topmostView, 
						new ChangeEvent(new MetaDataChangedReason(masterMovieView, ((MetaDataView) masterMovieView).getMetaData())));
			}
		}
			
		masterMovieView.setCurrentFrame(masterMovieView.getCurrentFrameNumber() + 1, new ChangeEvent());
		
		progressBar.setValue(masterMovieView.getCurrentFrameNumber());
	}

	/**
	 * {@inheritDoc}
	 */
	public void showDialog() {
		pack();
        setSize(getPreferredSize());
        setLocationRelativeTo(ImageViewerGui.getMainFrame());
        cmdExport.setEnabled(true);
        setVisible(true);
	}
	
	/**
	 * Deletes all resources and closes the dialog.
	 */
	public synchronized void finish() {
		// Destroy all threads still running
		try {
			if(exportThread != null) {
				exportThread.interrupt();
				exportThread.join();
				exportThread = null;
			}
			if(initThread != null) {
				initThread.interrupt();
				initThread.join();
				initThread = null;
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
		// Delete linked movie manager to unlink all movies
		if(linkedMovieManagerInstance > 0) {
			LinkedMovieManager.deleteInstance(linkedMovieManagerInstance);
			linkedMovieManagerInstance = -1;
		}

		// delete view chain
		if(topmostView != null) {
			topmostView.removeViewListener(this);
			
			// Remove all layers from LayeredView, so that
			// JHVJP2View.abolish() is called correctly
			LayeredView layeredView = topmostView.getAdapter(LayeredView.class);
			if(layeredView != null) {
				while(layeredView.getNumLayers() > 0) {
					layeredView.removeLayer(0);
				}
			}
			topmostView = null;
		}
		
		masterMovieView = null;
		
		// hide dialog
		setVisible(false);
	}
	
	/** Initializes the copy of the view chain.
	 * 
	 * This code is put into a separate function to
	 * be able to run it in a new thread. As a result,
	 * the application will not block the whole time.
	 */
	private void initViewchain() {
		while(!Thread.interrupted()) {
			
			// Get topmost view, which is not the ComponentView
			View topmostView = imagePanel.getView().getView();
			SubimageDataView topmostSubimageView = imagePanel.getView().getAdapter(SubimageDataView.class);
			this.topmostView = topmostSubimageView;
			
			// If viewport is correct, go on
			if(topmostSubimageView.getSubimageData() != null 
					&& topmostSubimageView.getSubimageData().getWidth() == imagePanel.getViewport().getWidth()) {
				
				// Give other layers a chance to finish
				try {
					Thread.sleep(500);
				} catch (InterruptedException e) {
					return;
				}
				
				// unplug topmost view from ComponentView
				topmostView.removeViewListener(imagePanel.getView());
				
				// Add SynchronizeView, to handle region selections
				SynchronizeOverviewChainView synchronizeView = new SynchronizeOverviewChainView();
				imagePanel.getView().setView(synchronizeView);
				
				// Add view dummy to manage region input
				viewDummy = new ViewDummy();
				synchronizeView.setObservedView(viewDummy);
				
				viewDummy.setViewport(StaticViewport.createAdaptedViewport(
						(Integer) txtWidth.getValue(), (Integer) txtHeight.getValue()), new ChangeEvent());
				viewDummy.setRegion(ViewHelper.cropRegionToImage(
						ImageViewerGui.getSingletonInstance().getMainView().getAdapter(RegionView.class).getRegion(),
						viewDummy.getMetaData()), new ChangeEvent());
				updateZoom();

				break;
			}
				
			try {
				Thread.sleep(50);
			} catch (InterruptedException e) {
				return;
			}
		}
	}
	
	/**
	 * Updates the zoom slider.
	 */
	private void updateZoom() {
		Vector2dDouble zoomVector = 
			viewDummy.getRegion().getSize().invertedScale(
			viewDummy.getMetaData().getPhysicalRectangle().getSize());
		
		double zoom = Math.max(zoomVector.getX(), zoomVector.getY());
		
		zoomSlider.setValue((int) (100 * zoom));
	}
	
	/**
	 * Starts exporting the movies.
	 */
	private void exportMovie() {
		
		// Disable all GUI elements
		txtTargetFile.setEnabled(false);
		cmdBrowseTargetFile.setEnabled(false);
		txtHeight.setEnabled(false);
		txtWidth.setEnabled(false);
		zoomSlider.setEnabled(false);
		loadFirstCheckBox.setEnabled(false);
		useDifferentialRotationTracking.setEnabled(false);
		speedSpinner.setEnabled(false);
		cmdExport.setEnabled(false);
		masterMovieView = null;
		
		// If selected file filter does not fit txtTargetFile, select a new one
		if(!txtTargetFile.getText().toLowerCase().endsWith(selectedOutputFormat.getExtension())) {
			if(txtTargetFile.getText().lastIndexOf("/") >= txtTargetFile.getText().lastIndexOf(".")) {
				txtTargetFile.setText(txtTargetFile.getText() + selectedOutputFormat.getExtension());
			} else {
				String extension = txtTargetFile.getText().toLowerCase().substring(
						txtTargetFile.getText().lastIndexOf("."), txtTargetFile.getText().length());
				
				if(extension.equals(".mp4")) {
					selectedOutputFormat = new MP4Filter();
				} else if(extension.equals(".mov")) {
					selectedOutputFormat = new MOVFilter();
				} else if(extension.equals(".png")) {
					selectedOutputFormat = new PNGSetFilter();
				} else if(extension.equals(".jpg") || extension.equals(".jpeg")) {
					selectedOutputFormat = new JPEGSetFilter();
				} else if(extension.equals(".zip")) {
					selectedOutputFormat = new PNGArchiveFilter();
				}
			}		
		}
		
		// Create new linked movie manager instance
		linkedMovieManagerInstance = LinkedMovieManager.createNewInstance();
		LinkedMovieManager.setActiveInstance(linkedMovieManagerInstance);
		
		// Get layered view
		LayeredView layeredView = topmostView.getAdapter(LayeredView.class);

		for(int i=0; i<layeredView.getNumLayers(); i++) {
			JHVJP2View jp2View = layeredView.getLayer(i).getAdapter(JHVJP2View.class);
			if(jp2View != null) {
				if(loadFirstCheckBox.isSelected()) {				
					jp2View.setReaderMode(ReaderMode.ONLYFIREONCOMPLETE);
					jp2View.setPersistent(true);
				} else {
					jp2View.setReaderMode(ReaderMode.NEVERFIRE);
				}
			}
			
			// Search all MovieViews
			if(masterMovieView == null && !(layeredView.getLayer(i).getAdapter(MovieView.class) instanceof TimedMovieView)) {
				masterMovieView = layeredView.getLayer(i).getAdapter(MovieView.class);
				
			} else {
				TimedMovieView timedMovieView = layeredView.getLayer(i).getAdapter(TimedMovieView.class);
				
				if(timedMovieView != null && timedMovieView.getMaximumFrameNumber() > 0) {
					
					MetaData metaData = ((MetaDataView) timedMovieView).getMetaData();
					if(metaData instanceof ObserverMetaData) {
						
						while(timedMovieView.getMaximumAccessibleFrameNumber() <= 0) {
							try {
								Thread.sleep(200);
							} catch (InterruptedException e) {
								return;
							}
						}
						
						// Set frame to wrong frame to force view to update on starting the export
						timedMovieView.setCurrentFrame(timedMovieView.getMaximumAccessibleFrameNumber(), new ChangeEvent());
						currentViewStatus.put(timedMovieView, new StatusStruct());
						
						// Link all movies
						timedMovieView.linkMovie();
						
						if(LinkedMovieManager.getActiveInstance().isMaster(timedMovieView)) {
							masterMovieView = timedMovieView;
						}
					}
				}
			}
		}
		
		if(masterMovieView == null) {
			JHVGlobals.warn("Warning", "Nothing to export: No layer contains an image series.");
			finish();
			return;
		}
		
		currentFrame = 1;
		
		progressBar.setMaximum(masterMovieView.getMaximumFrameNumber());

		// Set region and viewport
		topmostView.getAdapter(RegionView.class).setRegion(viewDummy.getRegion(), new ChangeEvent());
		topmostView.getAdapter(ViewportView.class).setViewport(viewDummy.getViewport(), new ChangeEvent());
		topmostView.getAdapter(RegionView.class).setRegion(viewDummy.getRegion(), new ChangeEvent());
		
		// Delete all files in JHV/temp
		File[] tempFiles = JHVDirectory.TEMP.getFile().listFiles(new FilenameFilter() {
			public boolean accept(File dir, String name) {
				return (name.contains("frame")
						&& (name.toLowerCase().endsWith(".bmp")
						||  name.toLowerCase().endsWith(".jpg")
						||  name.toLowerCase().endsWith(".png")));
			}
		});
		
		for(File tempFile : tempFiles) {
			tempFile.delete();
		}
		
		// Wait for all layers to finish decoding their last frame
		while(true) {
			boolean allSet = true;
			for(TimedMovieView movieView : currentViewStatus.keySet()) {
				if(((ObserverMetaData) ((MetaDataView) movieView).getMetaData()).getDateTime().compareTo(movieView.getCurrentFrameDateTime()) != 0) {
					allSet = false;
				}
			}
			
			if(allSet) {
				break;
			}
			
			try {
				Thread.sleep(200);
			} catch (InterruptedException e) {
				return;
			}
		}
		
		
		if(useDifferentialRotationTracking.isSelected()) {
			MainImagePanelMouseFocusController focusController = new MainImagePanelMouseFocusController();
			focusController.setView(topmostView);
			focusController.detach();
			additionalListener = focusController;
		}
		
		// Insert this as listener (and thus topmost element)
		topmostView.addViewListener(this);
		
		// Start playing
		masterMovieView.setCurrentFrame(0, new ChangeEvent());
	}
	
	/** A dummy to manage viewport, region and meta data during selection.
	 * 
	 * <p>This class is only used between initializing the view chain and
	 * starting the export.
	 * 
	 * @author Markus Langenberg
	 */
	private class ViewDummy extends AbstractView implements ViewportView, RegionView, MetaDataView {

		private Region region;
		private Viewport viewport;
		private MetaData metaData;
		
		/**
		 * Default constructor.
		 */
		public ViewDummy() {
			MetaDataView metaDataView = topmostView.getAdapter(MetaDataView.class);
			
			if(metaDataView != null) {
				metaData = metaDataView.getMetaData();
				region = metaData.getPhysicalRegion();
			}
		}
		
		/**
		 * {@inheritDoc}
		 */
		public Viewport getViewport() {
			return viewport;
		}

		/**
		 * {@inheritDoc}
		 */
		public boolean setViewport(Viewport v, ChangeEvent event) {
			viewport = v;
			region = ViewHelper.expandRegionToViewportAspectRatio(viewport, region, metaData);
			region = ViewHelper.contractRegionToViewportAspectRatio(viewport, region, metaData);
			
			event.addReason(new SubImageDataChangedReason(this));
			event.addReason(new ViewportChangedReason(this, viewport));
			notifyViewListeners(event);
			
			return true;
		}

		/**
		 * {@inheritDoc}
		 */
		@SuppressWarnings("unchecked")
		public <T extends View> T getAdapter(Class<T> c) {
			if (c.isInstance(this)) {
	            return (T) this;
	        } else {
	            return null;
	        }
		}

		/**
		 * {@inheritDoc}
		 */
		public Region getRegion() {
			return region;
		}

		/**
		 * {@inheritDoc}
		 */
		public boolean setRegion(Region r, ChangeEvent event) {
			region = r;
			region = ViewHelper.expandRegionToViewportAspectRatio(viewport, region, metaData);
			
			double viewportRatio = viewport.getAspectRatio();
			if(Math.abs(region.getWidth() / region.getHeight() - viewportRatio) > 0.01f) {
				region = StaticRegion.createAdaptedRegion(region.getRectangle()
		                .contractToAspectRatioKeepingCenter(viewportRatio));
			}
			
			event.addReason(new SubImageDataChangedReason(this));
			event.addReason(new RegionChangedReason(this, region));
			notifyViewListeners(event);
			updateZoom();
			
			return true;
		}

		/**
		 * {@inheritDoc}
		 */
		public MetaData getMetaData() {
			return metaData;
		}
	}
	
	/** File filter for movies, also providing the extension of the selected format.
	 * 
	 * @author Markus Langenberg
	 */
	private abstract class MovieFileFilter extends FileFilter {
		
		/** Returns the extension associated with this movie format.
		 * 
		 * @return Extension associated with this movie format
		 */
		public abstract String getExtension();
		
		/** Returns the extension used for the intermediate images.
		 * 
		 * @return Extension used for the intermediate images
		 */
		public abstract String getIntermediateExtension();
		
		/** Returns, whether intermediate images should be written
		 * to the temporary directory or to the final destination.
		 * 
		 * @return If true, intermediate images are written to the
		 * temporary directoy
		 */
		public abstract boolean useTempDirectoryForIntermediateImages();
	}
	
	/** Implementation of MovieFileFiter for Quicktime movies.
	 * 
	 * @author Markus Langenberg
	 */
	private class MOVFilter extends MovieFileFilter {

		/**
		 * {@inheritDoc}
		 */
		public boolean accept(File f) {
			return f.getName().toLowerCase().endsWith(".mov");
		}

		/**
		 * {@inheritDoc}
		 */
		public String getDescription() {
			return "Quicktime (.mov)";
		}
		
		/**
		 * {@inheritDoc}
		 */
		public String getExtension() {
			return ".mov";
		}
		
		/**
		 * {@inheritDoc}
		 */
		public String getIntermediateExtension() {
			return ".bmp";
		}

		/**
		 * {@inheritDoc}
		 */
		public boolean useTempDirectoryForIntermediateImages() {
			return true;
		}
	}
	
	/** Implementation of MovieFileFilter for MPEG-4 movies.
	 * 
	 * @author Markus Langenberg
	 */
	private class MP4Filter extends MovieFileFilter {

		/**
		 * {@inheritDoc}
		 */
		public boolean accept(File f) {
			return f.getName().toLowerCase().endsWith(".mp4");
		}

		/**
		 * {@inheritDoc}
		 */
		public String getDescription() {
			return "MPEG-4 (.mp4)";
		}
		
		/**
		 * {@inheritDoc}
		 */
		public String getExtension() {
			return ".mp4";
		}
		
		/**
		 * {@inheritDoc}
		 */
		public String getIntermediateExtension() {
			return ".bmp";
		}

		/**
		 * {@inheritDoc}
		 */
		public boolean useTempDirectoryForIntermediateImages() {
			return true;
		}
	}
	
	/** Implementation of MovieFileFilter for a set of JPEG images.
	 * 
	 * @author Markus Langenberg
	 */
	private class JPEGSetFilter extends MovieFileFilter {

		/**
		 * {@inheritDoc}
		 */
		public boolean accept(File f) {
			return f.getName().toLowerCase().endsWith(".jpg")
				|| f.getName().toLowerCase().endsWith(".jpeg");
		}

		/**
		 * {@inheritDoc}
		 */
		public String getDescription() {
			return "Set of JPG images (.jpg)";
		}
		
		/**
		 * {@inheritDoc}
		 */
		public String getExtension() {
			return ".jpg";
		}
		
		/**
		 * {@inheritDoc}
		 */
		public String getIntermediateExtension() {
			return ".jpg";
		}

		/**
		 * {@inheritDoc}
		 */
		public boolean useTempDirectoryForIntermediateImages() {
			return false;
		}
	}
	
	/** Implementation of MovieFileFilter for a set of PNG images.
	 * 
	 * @author Markus Langenberg
	 */
	private class PNGSetFilter extends MovieFileFilter {

		/**
		 * {@inheritDoc}
		 */
		public boolean accept(File f) {
			return f.getName().toLowerCase().endsWith(".png");
		}

		/**
		 * {@inheritDoc}
		 */
		public String getDescription() {
			return "Set of PNG images (.png)";
		}
		
		/**
		 * {@inheritDoc}
		 */
		public String getExtension() {
			return ".png";
		}
		
		/**
		 * {@inheritDoc}
		 */
		public String getIntermediateExtension() {
			return ".png";
		}

		/**
		 * {@inheritDoc}
		 */
		public boolean useTempDirectoryForIntermediateImages() {
			return false;
		}
	}
	
	/** Implementation of MovieFileFilter for ZIP archives of JPEG images.
	 * 
	 * @author Markus Langenberg
	 */
	private class JPEGArchiveFilter extends MovieFileFilter {

		/**
		 * {@inheritDoc}
		 */
		public boolean accept(File f) {
			return f.getName().toLowerCase().endsWith(".zip");
		}

		/**
		 * {@inheritDoc}
		 */
		public String getDescription() {
			return "ZIP archive of JPG images (.zip)";
		}
		
		/**
		 * {@inheritDoc}
		 */
		public String getExtension() {
			return ".zip";
		}
		
		/**
		 * {@inheritDoc}
		 */
		public String getIntermediateExtension() {
			return ".jpg";
		}

		/**
		 * {@inheritDoc}
		 */
		public boolean useTempDirectoryForIntermediateImages() {
			return true;
		}
	}
	
	/** Implementation of MovieFileFilter for ZIP archives of PNG images.
	 * 
	 * @author Markus Langenberg
	 */
	private class PNGArchiveFilter extends MovieFileFilter {

		/**
		 * {@inheritDoc}
		 */
		public boolean accept(File f) {
			return f.getName().toLowerCase().endsWith(".zip");
		}

		/**
		 * {@inheritDoc}
		 */
		public String getDescription() {
			return "ZIP archive of PNG images (.zip)";
		}
		
		/**
		 * {@inheritDoc}
		 */
		public String getExtension() {
			return ".zip";
		}
		
		/**
		 * {@inheritDoc}
		 */
		public String getIntermediateExtension() {
			return ".png";
		}

		/**
		 * {@inheritDoc}
		 */
		public boolean useTempDirectoryForIntermediateImages() {
			return true;
		}
	}
}