package org.helioviewer.jhv.internal_plugins.filter.SOHOLUTFilterPlugin;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.URL;

import org.helioviewer.jhv.JHVDirectory;

/**
 * Collection of color tables for 16-bit images.
 */
public class SOHOLUT16 {

	/**
	 * Available lookup tables
	 */
    public enum LUT {
        GRAY("ctable_idl_16bit_00_Black-White_Linear"),
        
        EUVI_171("ctable_idl_16bit_EUVI_171"),
        EUVI_195("ctable_idl_16bit_EUVI_195"),
        EUVI_284("ctable_idl_16bit_EUVI_284"),
        EUVI_304("ctable_idl_16bit_EUVI_304"),

        EIT_171("ctable_idl_16bit_EIT_171"),
        EIT_195("ctable_idl_16bit_EIT_195"),
        EIT_284("ctable_idl_16bit_EIT_284"),
        EIT_304("ctable_idl_16bit_EIT_304"),
        
        BlueWhiteLinear("ctable_idl_16bit_01_Blue-White_Linear"),
        RedTemperature("ctable_idl_16bit_03_Red_Temperature"),
        BGRY("ctable_idl_16bit_04_Blue-Green-Red-Yellow"),
        GreenWhiteLinear("ctable_idl_16bit_08_Green-White_Linear"),
        GreenWhiteExponential("ctable_idl_16bit_09_Green-White_Exponential"),
        Rainbow1("ctable_idl_16bit_13_Rainbow_1"),
        BlueRed("ctable_idl_16bit_33_Blue-Red"),
        Rainbow2("ctable_idl_16bit_34_Rainbow_2");

        private final String lut;

        /**
         * Default constructor.
         * 
         * @param ints Filename of the text file containing the lookup data 
         */
        LUT(String ints) {
            lut = ints;
        }

        /** Returns the filename of the text file containing the lookup data.
         * 
         * @return Filename of the text file containing the lookup data
         */
        private String getLUT() {
            return lut;
        }
    }

    /** Fills the output array with values based on the given
	 * input data using the given lookup table.
	 * 
	 * The output data must have the same size as the input data
	 * to call this function.
	 * 
	 * @param shortBuf Input data
	 * @param intBuf Output data
	 * @param lut Lookup table to use
	 */
    public static void lookup16(short[] shortBuf, int[] intBuf, SOHOLUT.LUT lut) {
        lut.name();
        int[] intLUT = getContents(lut.name());
        if(lut.isInverted()) {
        	int offset = intLUT.length - 1;
        	for (int i = 0; i < shortBuf.length; ++i) {
        		intBuf[i] = intLUT[offset - (shortBuf[i] & 0xFFFF)];
        	}
        } else {
        	for (int i = 0; i < shortBuf.length; ++i) {
        		intBuf[i] = intLUT[(shortBuf[i] & 0xFFFF)];
        	}
        }
        shortBuf = null;
    }

    /**
     * Fetch the entire contents of a text file containing integer values.
     * 
     * The Strings are parsed to integer values and returned as a integer
     * array.
     * 
     * @param lut Lookup table to read
     */
    public static int[] getContents(String lut) {
        String nameString = LUT.valueOf(lut).getLUT();
        try {

        	URL source = SOHOLUT16.class.getResource("luts" + JHVDirectory.FILE_SEP + nameString);
            BufferedReader br = new BufferedReader(new InputStreamReader(source.openStream()));
            String strLine;

            int[] intArray = new int[65537];
            int counter = 0;
            while ((strLine = br.readLine()) != null) {

                String[] array = strLine.split(" ");

                for (int i = 0; i < array.length; i++) {
                    if (array[i].equals(" ") || array[i].equals("")) {

                    } else {
                        intArray[counter] = Integer.parseInt(array[i]);
                        counter++;
                    }

                }

            }

            br.close();

            return intArray;
        } catch (Exception e) {// Catch exception if any
            e.printStackTrace();
            return null;
        }
    }
}
