package org.helioviewer.jhv.internal_plugins.filter.SOHOLUTFilterPlugin;

import org.helioviewer.viewmodel.filter.Filter;
import org.helioviewer.viewmodel.imageformat.SingleChannelImageFormat;
import org.helioviewer.viewmodel.metadata.HelioviewerMetaData;
import org.helioviewer.viewmodel.metadata.MetaData;
import org.helioviewer.viewmodel.view.FilterView;
import org.helioviewer.viewmodel.view.MetaDataView;
import org.helioviewer.viewmodel.view.SubimageDataView;
import org.helioviewer.viewmodelplugin.filter.FilterContainer;
import org.helioviewer.viewmodelplugin.filter.FilterTab;
import org.helioviewer.viewmodelplugin.filter.FilterTabList;
import org.helioviewer.viewmodelplugin.filter.FilterTabPanelManager;
import org.helioviewer.viewmodelplugin.filter.FilterTabDescriptor.Type;
import org.helioviewer.viewmodelplugin.filter.FilterTabPanelManager.Area;

/** Filter plugin for applying a color table to single channel images.
 * 
 * <p>This plugin provides several different color tables to improve
 * the visual impression of single channel images. It manages a
 * filter for applying the color table and a combobox to change it.
 *
 */
public class SOHOLUTFilterPlugin extends FilterContainer  {
    
    /**
     * {@inheritDoc}
     */
    public String getDescription() {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    public String getName() {
        return "Color Tables";
    }

    /** 
     * 
     * If the given meta data contain helioviewer meta data, the
     * default color table for the given instrument is chosen.
     * Otherwise, the color table is set to gray by default, which
     * does not change the color at all. 
     * 
     * 
     */
    @Override
    protected void installFilterImpl(FilterView filterView, FilterTabList tabList) {
        
        if (!(filterView.getAdapter(SubimageDataView.class).getSubimageData().getImageFormat() instanceof SingleChannelImageFormat))
            return;
        
        MetaDataView metaDataView = filterView.getAdapter(MetaDataView.class);
        
        if (metaDataView == null)
            return;
        
        MetaData metaData = metaDataView.getMetaData();
        
        if (metaData == null || !(metaData instanceof HelioviewerMetaData))
            return;
        
        HelioviewerMetaData hvMetaData = (HelioviewerMetaData)metaData;
        
        SOHOLUT.LUT lut = SOHOLUT.LUT.GRAY;
        
        if(hvMetaData != null) {
            if(hvMetaData.getInstrument().equalsIgnoreCase("EIT")) {
                if(hvMetaData.getMeasurement().equalsIgnoreCase("171")) {
                    lut = SOHOLUT.LUT.EIT_171;
                } else if(hvMetaData.getMeasurement().equalsIgnoreCase("195")) {
                    lut = SOHOLUT.LUT.EIT_195;
                } else if(hvMetaData.getMeasurement().equalsIgnoreCase("284")) {
                    lut = SOHOLUT.LUT.EIT_284;
                } else if(hvMetaData.getMeasurement().equalsIgnoreCase("304")) {
                    lut = SOHOLUT.LUT.EIT_304;
                }
            } else if (hvMetaData.getInstrument().equalsIgnoreCase("LASCO")) {
                if(hvMetaData.getDetector().equalsIgnoreCase("C2")) {
                    lut = SOHOLUT.LUT.RedTemperature;
                } else if(hvMetaData.getDetector().equalsIgnoreCase("C3")) {
                    lut = SOHOLUT.LUT.BlueWhiteLinear;
                }
            }
        }
        
        SOHOLUTFilter filter = new SOHOLUTFilter(lut);
        filterView.setFilter(filter);
        
        FilterTabPanelManager manager = tabList.getFirstPanelManagerByType(Type.DEFAULT_FILTER);
        
        if (manager == null) {
            manager = new FilterTabPanelManager();
            tabList.add(new FilterTab(Type.COSTUM, getName(), manager));
        }
        
        SOHOLUTPanel pane = new SOHOLUTPanel();
        pane.setFilter(filter);
        manager.add(pane, Area.BOTTOM);        
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Class<? extends Filter> getFilterClass() {
        return SOHOLUTFilter.class;
    }
}
