package org.helioviewer.jhv.internal_plugins.filter.SOHOLUTFilterPlugin;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.Icon;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JToggleButton;
import javax.swing.border.BevelBorder;

import org.helioviewer.viewmodel.filter.Filter;
import org.helioviewer.viewmodelplugin.filter.FilterPanel;
import org.helioviewer.viewmodelplugin.filter.FilterTabPanelManager.Area;
import org.helioviewer.jhv.gui.IconBank;
import org.helioviewer.jhv.gui.IconBank.JHVIcon;

/**
 * Panel containing a combobox for choosing the color table.
 */
public class SOHOLUTPanel extends FilterPanel implements ActionListener {

	private static final long serialVersionUID = 1L;
	
	private static final Icon invertIcon = IconBank.getIcon(JHVIcon.INVERT);

	private SOHOLUTFilter filter;

    char angstrom = '\u212B';
    String[] tableList = { "Gray", "STEREO EUVI 171 " + angstrom,
            "STEREO EUVI 284 " + angstrom, "STEREO EUVI 195 " + angstrom,
            "STEREO EUVI 304 " + angstrom, "SOHO EIT 171 " + angstrom,
            "SOHO EIT 284 " + angstrom, "SOHO EIT 195 " + angstrom,
            "SOHO EIT 304 " + angstrom, "Blue/White Linear",
            "Red Temperature", "Blue/Green/Red/Yellow",
            "Green/White Linear", "Green/White Exponential", "Rainbow 1",
            "Blue/Red", "Rainbow 2" };

    final SOHOLUT.LUT[] lutList = { SOHOLUT.LUT.GRAY, SOHOLUT.LUT.EUVI_171,
            SOHOLUT.LUT.EUVI_284, SOHOLUT.LUT.EUVI_195,
            SOHOLUT.LUT.EUVI_304, SOHOLUT.LUT.EIT_171, SOHOLUT.LUT.EIT_284,
            SOHOLUT.LUT.EIT_195, SOHOLUT.LUT.EIT_304,
            SOHOLUT.LUT.BlueWhiteLinear, SOHOLUT.LUT.RedTemperature,
            SOHOLUT.LUT.BGRY, SOHOLUT.LUT.GreenWhiteLinear,
            SOHOLUT.LUT.GreenWhiteExponential, SOHOLUT.LUT.Rainbow1,
            SOHOLUT.LUT.BlueRed, SOHOLUT.LUT.Rainbow2 };

    private JComboBox combobox = new JComboBox(tableList);
    private JToggleButton invertButton = new JToggleButton(invertIcon);

    /** Default constructor.
     * 
     * @param newFilter Filter which is controlled by this panel
     */
    public SOHOLUTPanel() {
    	setLayout(new BoxLayout(this, BoxLayout.LINE_AXIS));
    	
    	JLabel title = new JLabel("Color:");
        title.setPreferredSize(new Dimension(FilterPanel.titleWidth, FilterPanel.height));
        add(title);
		
		combobox.setToolTipText("Choose a color table");
		combobox.setPreferredSize(new Dimension(150, combobox.getPreferredSize().height));
        combobox.addActionListener(this);
        add(combobox);
        
        add(Box.createHorizontalStrut(14));
        
        invertButton.setToolTipText("Invert color table");
        invertButton.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
        invertButton.setPreferredSize(new Dimension(FilterPanel.valueWidth-14, FilterPanel.height));
        invertButton.addActionListener(this);
        add(invertButton);
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public void setFilter(Filter filter) {
        if (filter instanceof SOHOLUTFilter)
            this.filter = (SOHOLUTFilter)filter;
        
        for(int i=0; i< lutList.length; i++) {
            if(lutList[i] == this.filter.getLUT()) {
                combobox.setSelectedIndex(i);
                break;
            }   
        }
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public Area getArea() {
        return Area.BOTTOM;
    }

    /**
     * Sets the color table
     */
    public void actionPerformed(ActionEvent e) {
    	filter.setLUT(lutList[combobox.getSelectedIndex()], invertButton.isSelected());
    	
    	if(e.getSource() == invertButton) {
	    	if(invertButton.isSelected()) {
	    		invertButton.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
	    	} else {
	    		invertButton.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
	    	}
    	}
    }
}
