package org.helioviewer.jhv.internal_plugins.filter.opacity;

import java.awt.Dimension;

import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.helioviewer.viewmodel.filter.Filter;
import org.helioviewer.viewmodelplugin.filter.FilterPanel;
import org.helioviewer.viewmodelplugin.filter.FilterTabPanelManager.Area;

/** Panel containing a slider for changing the opacity of the image.
 * 
 * @author Markus Langenberg
 */
public class OpacityPanel extends FilterPanel implements ChangeListener {

	private static final long serialVersionUID = 1L;
	
	private JSlider opacitySlider;
	private JLabel opacityLabel;
	private OpacityFilter filter;
	
	/** Default constructor.
     * 
     * @param newFilter Filter which is controlled by this panel
     */
    public OpacityPanel() {
		setLayout(new BoxLayout(this, BoxLayout.LINE_AXIS));
		
		JLabel title = new JLabel("Opacity:");
        title.setPreferredSize(new Dimension(FilterPanel.titleWidth, FilterPanel.height));
        add(title);
		
		opacitySlider = new JSlider(JSlider.HORIZONTAL, 0, 100, 100);
		opacitySlider.setMajorTickSpacing(20);
		opacitySlider.setPaintTicks(true);
		opacitySlider.setPreferredSize(new Dimension(150, opacitySlider.getPreferredSize().height));
		opacitySlider.addChangeListener(this);
		add(opacitySlider);
		
		opacityLabel = new JLabel(opacitySlider.getValue() + "%");
		opacityLabel.setHorizontalAlignment(JLabel.RIGHT);
		opacityLabel.setPreferredSize(new Dimension(FilterPanel.valueWidth, FilterPanel.height));
        add(opacityLabel);  
	}
    
    /**
     * {@inheritDoc}
     */
    @Override
    public void setFilter(Filter filter) {
        if (filter instanceof OpacityFilter) {
            this.filter = (OpacityFilter)filter;
            this.filter.setOpacityPanel(this);
        }
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public Area getArea() {
        return Area.TOP;
    }

    /**
     * Sets the opacity of the image.
     */
	public void stateChanged(ChangeEvent e) {
		filter.setOpacity((float) opacitySlider.getValue() / 100);
		opacityLabel.setText(opacitySlider.getValue() + "%");
	}

	/** Sets the slider value.
	 * 
	 * This may be useful, if the opacity from another source
	 * than the slider itself.
	 * 
	 * @param value New slider value. Must be within [0, 100]
	 */
	public void setSliderValue(int value) {
	    
	    if (value >= opacitySlider.getMinimum() && value <= opacitySlider.getMaximum())
    	    opacitySlider.setValue(value);
	}
}
