package org.helioviewer.jhv.internal_plugins.filter.sharpen;

import java.awt.Dimension;

import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.helioviewer.viewmodel.filter.Filter;
import org.helioviewer.viewmodelplugin.filter.FilterPanel;
import org.helioviewer.viewmodelplugin.filter.FilterTabPanelManager.Area;

/** Panel containing a slider for changing the weighting of the sharpening.
 * 
 * @author Markus Langenberg
 */
public class SharpenPanel extends FilterPanel implements ChangeListener {

	private static final long serialVersionUID = 1L;
	
	private JSlider sharpeningSlider;
	private JLabel sharpeningLabel;
	private SharpenFilter filter;
	
	/** Default constructor.
     * 
     * @param newFilter Filter which is controlled by this panel
     */
    public SharpenPanel() {
		setLayout(new BoxLayout(this, BoxLayout.LINE_AXIS));
		
		JLabel title = new JLabel("Sharpen:");
        title.setPreferredSize(new Dimension(FilterPanel.titleWidth, FilterPanel.height));
        add(title);
		
		sharpeningSlider = new JSlider(JSlider.HORIZONTAL, 0, 100, 0);
		sharpeningSlider.setMajorTickSpacing(20);
		sharpeningSlider.setPaintTicks(true);
		sharpeningSlider.setPreferredSize(new Dimension(150, sharpeningSlider.getPreferredSize().height));
		sharpeningSlider.addChangeListener(this);
		add(sharpeningSlider);
		
		sharpeningLabel = new JLabel("0%");
		sharpeningLabel.setHorizontalAlignment(JLabel.RIGHT);
		sharpeningLabel.setPreferredSize(new Dimension(FilterPanel.valueWidth, FilterPanel.height));
        add(sharpeningLabel);  
	}
    
    /**
     * {@inheritDoc}
     */
    @Override
    public void setFilter(Filter filter) {
        if (filter instanceof SharpenFilter)
            this.filter = (SharpenFilter)filter;
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public Area getArea() {
        return Area.TOP;
    }

	/**
	 * Sets the weighting of the sharpening.
	 */
    public void stateChanged(ChangeEvent e) {
		filter.setWeighting((float)sharpeningSlider.getValue() / 10.0f);
		sharpeningLabel.setText(sharpeningSlider.getValue() + "%");
	}
}
