package org.helioviewer.jhv.internal_plugins.overlay.solarevents;

import org.helioviewer.viewmodel.renderer.physical.PhysicalRenderer;
import org.helioviewer.viewmodel.view.OverlayView;
import org.helioviewer.viewmodelplugin.overlay.OverlayContainer;
import org.helioviewer.viewmodelplugin.overlay.OverlayControlComponent;
import org.helioviewer.viewmodelplugin.overlay.OverlayControlComponentManager;

/**
 * Overlay plug-in to display different kinds of solar events.
 * <p>
 * This plug-in provides several different solar events. The different solar
 * events can be managed by a corresponding control component. The component
 * allows to choose solar events from a combobox. Already chosen solar events
 * are displayed in a list and additional buttons to control the selected events
 * are displayed below the combobox.
 * 
 * @author Stephan Pagel
 */
public class SolarEventPlugin extends OverlayContainer {

    /**
     * {@inheritDoc}
     */
    @Override
    public Class<? extends PhysicalRenderer> getOverlayClass() {
        return SolarEventRenderer.class;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void installOverlayImpl(OverlayView overlayView, OverlayControlComponentManager controlList) {
        
        SolarEventCatalogs.getSingletonInstance().loadCatalogs();
        
        overlayView.setRenderer(new SolarEventRenderer());
        controlList.add(new OverlayControlComponent(new SolarEventCatalogsPanel(), getName()));
    }

    /**
     * {@inheritDoc}
     */
    public String getDescription() {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    public String getName() {
        return "Solar Events";
    }

}
