package org.helioviewer.jhv.opengl;

import javax.media.opengl.GL;

import org.helioviewer.jhv.gui.ImageViewerGui;
import org.helioviewer.viewmodel.view.opengl.GLTextureHelper;

/** Class to check and manage some OpenGL properties.
 * 
 * @author Markus Langenberg
 *
 */
public class GLInfo {
	
	// data
	private static boolean glUsable = true;
	private static boolean glActivated = true;
	private static boolean glInitiated = false;
	private static float version;
	
	private GLInfo() {}
	
	/** Updates the OpenGL settings by reading the OpenGL properties.
	 * 
	 * By default, activates OpenGL if possible.
	 * 
	 * @param gl Valid reference to the current gl object
	 * @see #setGlEnabled(boolean)
	 */
	public static void update(GL gl) {
		String versionStr = gl.glGetString(GL.GL_VERSION);
		versionStr = versionStr.substring( 0, 3);
		
		version = new Float( versionStr ).floatValue();
		
		String extensionStr = gl.glGetString(GL.GL_EXTENSIONS);
		
		glUsable = (version >= 1.35f)
					&& extensionStr.contains("GL_ARB_multitexture")
					&& extensionStr.contains("GL_ARB_fragment_program");

		if(!glUsable && glActivated)
			glActivated = false;
		
		glInitiated = true;
		
		//GLTextureHelper.setTextureNonPowerOfTwo(gl.isExtensionAvailable("GL_ARB_texture_non_power_of_two"));
		GLTextureHelper.setTextureNonPowerOfTwo(false);
	}
	
	/** Returns, whether OpenGL is activated.
	 * 
	 * OpenGL can be deactivated for two reasons: The
	 * machine satisfies the minimal requirements of the
	 * user switched it of manually.
	 * 
	 * @return true, if OpenGL is activated, false otherwise
	 * @see #setGlEnabled(boolean)
	 */
	public static boolean glIsEnabled() {
		return glActivated;
	}
	
	/** Manually switches OpenGL on or off.
	 * 
	 * If the machine does not satisfy the minimal requirements
	 * for using OpenGL, this functions does nothing.
	 * 
	 * @param enabled true to use OpenGL, false otherwise
	 */
	public static void setGlEnabled(boolean enabled)
	{
		boolean before = glActivated;
		
		if(glUsable) {
			glActivated = enabled;
		}
		else {
			glActivated = false;
		}
		
		if(before != glActivated && glInitiated) {
			ImageViewerGui.getSingletonInstance().createViewchains();
		}
	}
	
	/** Returns the OpenGL version available on this machine.
	 * 
	 * @return OpenGL version
	 */
	public static float getVersion() {
		return version;
	}
}
