package org.helioviewer.jhv.opengl;

import java.awt.Dimension;

import javax.media.opengl.GL;
import javax.media.opengl.GLAutoDrawable;
import javax.media.opengl.GLCanvas;
import javax.media.opengl.GLEventListener;

import org.helioviewer.jhv.JHVDirectory;
import org.helioviewer.jhv.gui.ImageViewerGui;
import org.helioviewer.viewmodel.view.opengl.GLSharedContext;
import org.helioviewer.viewmodel.view.opengl.shader.GLShaderBuilder;
import org.helioviewer.viewmodel.view.opengl.shader.GLShaderHelper;

import com.sun.opengl.util.Animator;

/** Component to initialize OpenGL.
 * 
 * <p>The only purpose of this component is to be visible for
 * one moment during the startup sequence of JHV. It calls
 * {@link GLInfo#update(GL)} to get the application running
 * and initializes some OpenGL classes.
 * 
 * @author Markus Langenberg
 *
 */
public class GLInitPanel extends GLCanvas {
	
	private static final long serialVersionUID = 1L;

	/**
	 * Implementation GLEventHandler to startup OpenGL.
	 * It implements all listener functions.
	 */
	private class GLListener implements GLEventListener {
		private GLInitPanel parent;
		
		GLListener(GLInitPanel _parent) {
			super();
			parent = _parent;
		}
		
		public void display(GLAutoDrawable gLDrawable) {
			final GL gl = gLDrawable.getGL();
	        
	        gl.glClear(GL.GL_COLOR_BUFFER_BIT);      
		}

		public void displayChanged(GLAutoDrawable drawable, boolean modeChanged, boolean deviceChanged) {}

		public void init(GLAutoDrawable drawable) {
			parent.postInit();
			
			final GL gl = drawable.getGL();
			
			gl.glShadeModel(GL.GL_FLAT);
			gl.glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
			gl.glEnable(GL.GL_TEXTURE_1D);
			gl.glEnable(GL.GL_TEXTURE_2D);
			
			GLShaderHelper.initHelper(gl, JHVDirectory.LIBS.getPath(), JHVDirectory.TEMP.getPath());
			GLShaderBuilder.initShaderBuilder(gl);
			
			GLInfo.update(gl);
			
			ImageViewerGui.getSingletonInstance().createViewchains();
		}

		public void reshape(GLAutoDrawable drawable, int x, int y, int width, int height) {}
	}
	
	private Animator animator;
	
	/**
	 * Default constructor
	 */
	public GLInitPanel() {
		super(null, null, GLSharedContext.getSharedContext(), null);
		
		GLListener listener = new GLListener(this);
		addGLEventListener(listener);
		
		animator = new Animator(this);
		animator.start();
		
		setPreferredSize(new Dimension(1,1));
	}
	
	/** This function is called after OpenGL is initialzed.
	 * It disposes itself, since the component will not
	 * be used any more.
	 */
	private void postInit() {
		animator.stop();
		setVisible(false);
	}
}
