package org.helioviewer.viewmodel.changeevent;

import org.helioviewer.viewmodel.metadata.MetaData;
import org.helioviewer.viewmodel.view.View;

/**
 * Class represents a change reason when a layer of the layered view has changed.
 * 
 * @author Stephan Pagel
 * @author Markus Langenberg
 * */
public class MetaDataChangedReason implements ChangedReason {
	// ///////////////////////////////////////////////////////////////
	// Definitions
	// ///////////////////////////////////////////////////////////////

	// memorizes the associated view
	private View view;
	
	// memorizes the changed meta data
	private MetaData metaData;

	// ///////////////////////////////////////////////////////////////
	// Definitions
	// ///////////////////////////////////////////////////////////////

	/**
	 * Default constructor
	 * 
	 * @param aView
	 *            View which caused the change reason.
	 */
	public MetaDataChangedReason(View aView, MetaData aNewMetaData) {

		// memorize view
		view = aView;
		
		metaData = aNewMetaData;
	}

	/**
     * {@inheritDoc}
     */
	public View getView() {
		return view;
	}
	
	/**
	 * Returns the new meta data object which was defined when this change reason occurred.
	 * 
	 * @return new meta data object.
	 * */
	public MetaData getNewMetaData() {
		return metaData;
	}
}
