package org.helioviewer.viewmodel.io;

import java.net.URI;

import org.json.JSONException;
import org.json.JSONObject;

/** Wrapper for JSONObject.
 * 
 * The idea behind this class is simply to get a better name and
 * to prevent the user from handling all the JSONExceptions. 
 * 
 * @author Markus Langenberg
 */
public class APIResponse {
	
	URI uri;
	JSONObject data;

	/**
	 * Default constructor.
	 * 
	 * @param source string containing the formated JSON object
	 */
	public APIResponse(String source) {
		try {
			data = new JSONObject(source);
			uri = new URI(data.getString("uri"));
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/** Returns the URI of the image which results from this API response.
	 * 
	 * The URI is used as the unique identifier for the response.
	 *  
	 * @return unique URI corresponding to this response
	 */
	public URI getURI() {
		return uri;
	}
	
	/** Returns the value for a given key.
	 * 
	 * If the key does not exist, returns null.
	 * 
	 * @param key Key to search for
	 * @return value for given key
	 */
	public String getString(String key) {
		try {
			return data.getString(key);
		} catch (JSONException e) {}
		return null;
	}
}
