package org.helioviewer.viewmodel.io;

import java.net.URI;
import java.util.HashMap;

/** Dump for APIResponses.
 * 
 * This class stores API responses, to that other classes
 * all over the application can get additional information
 * provided by the API.
 * 
 * @author Markus Langenberg
 */
public class APIResponseDump {

	private static final APIResponseDump singletonInstance = new APIResponseDump();
	private HashMap<URI, APIResponse> dump = new HashMap<URI, APIResponse>();
	
	private APIResponseDump() {}
	
	/** Gets singleton object
	 * 
	 * @return
	 */
	public static APIResponseDump getSingletonInstance() {
		return singletonInstance;
	}
	
	/** Puts a new response on the dump.
	 * 
	 * @param newReponse new response
	 */
	public void putResponse(APIResponse newResponse) {
		dump.put(newResponse.getURI(), newResponse);
	}
	
	/** Gets the response corresponding to the URI.
	 * 
	 * The response is removed from the dump afterwards.
	 * 
	 * @param uri URI from corresponding response
	 * @return
	 */
	public APIResponse getResponse(URI uri) {
		return getResponse(uri, false);
	}
	
	/** Gets the response corresponding to the URI.
	 * 
	 * @param uri URI from corresponding response
	 * @param keepResponse if true, the response stays on the dump, otherwise it is removed
	 * @return
	 */
	public APIResponse getResponse(URI uri, boolean keepResponse) {
		APIResponse response = dump.get(uri);
		if(!keepResponse) {
			dump.remove(uri);
		}
		return response;
	}
}
