package org.helioviewer.viewmodel.metadata;

import org.helioviewer.base.physics.Constants;

/** Implementation of MetaData, extends HelioviewerMetaData.
 * 
 * <p>This special implementation also provides informations
 * about the occulting disc. The {@link MetaDataConstructor}
 * should only produce this implementation, if there is there
 * is actual information about the occulting disc present, so
 * it is possible to test this via instanceof.
 * 
 * @author Markus Langenberg
 *
 */
public class HelioviewerOcculterMetaData extends HelioviewerMetaData implements
		OcculterMetaData {
	
	private double innerRadius;
	private double outerRadius;

	/** Default constructor.
     * 
     * Tries to read all informations required.
     * 
     * @param m Meta data container serving as a base for the construction
     */
    public HelioviewerOcculterMetaData(MetaDataContainer m) {
		super(m);
		
		innerRadius = m.tryGetDouble("HV_ROCC_INNER") * Constants.SunRadius;
		outerRadius = m.tryGetDouble("HV_ROCC_OUTER") * Constants.SunRadius;
	}
    
    /**
     * Copy constructor
     * 
     * @param original Object to copy
     */
    public HelioviewerOcculterMetaData(HelioviewerOcculterMetaData original) {
    	super(original);
    	innerRadius = original.innerRadius;
    	outerRadius = original.outerRadius;
    }

	/**
	 * {@inheritDoc}
	 */
    public double getInnerPhysicalOcculterRadius() {
		return innerRadius;
	}

    /**
	 * {@inheritDoc}
	 */
    public double getOuterPhysicalOcculterRadius() {
		return outerRadius;
	}
}
