package org.helioviewer.viewmodel.metadata;

/** Factory for creating meta data out of a meta data container.
 * 
 * <p>This factory ensures, that the correct type of meta data is
 * generated. Currently, it supports {@link HelioviewerMetaData},
 * its extension {@link HelioviewerOcculterMetaData} and 
 * {@link PixelBasedMetaData} as a fallback solution.
 * 
 * @author Ludwig Schmidt
 *
 */
public class MetaDataConstructor {

	/** Returns an implementation of MetaData.
	 * 
	 * The function tries to search which implementation matches
	 * the image contents best.
	 * 
	 * @param mdc Meta data container serving as a base for the construction
	 * @return Implementation of MetaData
	 */
    public static MetaData getMetaData(MetaDataContainer mdc) {
    	
    	HelioviewerMetaData metaData;
    	
    	// If the key HV_ROCC_INNER, create HelioviewerOcculterMetaData
    	if(mdc.tryGetDouble("HV_ROCC_INNER") != 0.0) {
    		metaData = new HelioviewerOcculterMetaData(mdc);
    	
    	// Otherwise, try HelioviewerMetaData
    	} else {
    		metaData = new HelioviewerMetaData(mdc);
    	}
    	
    	// If the sun radius is -1, there wasn't any
    	// supported meta data available
        if (metaData.getSunPixelRadius() == -1) {
            return new PixelBasedMetaData(mdc.getPixelWidth(), mdc
                    .getPixelHeight());
        } else {
            return metaData;
        }
    }
}
