package org.helioviewer.viewmodel.renderer.screen;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;

/** Implementation of ScreenRenderGraphics operating on Java BufferedImages.
 * 
 * <p>Maps all methods to corresponding methods of Graphics2D.
 * 
 * @author Stephan Pagel
 * 
 */
public class BufferedImageScreenRenderGraphics extends AbstractScreenRenderGraphics {

	private Graphics2D graphics;
	
	/** Default constructor.
	 * 
	 * <p>The caller has to provide a Java Graphics2D object, which
	 * can be used by this renderer.
	 * 
	 * @param g Java Graphics2D object, that should be used for drawing.
	 */
	public BufferedImageScreenRenderGraphics(Graphics2D g) {
		graphics = g;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public void drawLine(int x0, int y0, int x1, int y1) {
		graphics.drawLine(x0, y0, x1, y1);
	}

	/**
	 * {@inheritDoc}
	 */
	public void drawRectangle(int x, int y, int width, int height) {
		graphics.drawRect(x, y, width, height);
	}

	/**
	 * {@inheritDoc}
	 */
	public void fillRectangle(int x, int y, int width, int height) {
		graphics.fillRect(x, y, width, height);		
	}
	
	/**
	 * {@inheritDoc}
	 */
	public void drawOval(int x, int y, int width, int height) {
		graphics.drawOval(x, y, width, height);
	}
	
	/**
	 * {@inheritDoc}
	 */
	public void fillOval(int x, int y, int width, int height) {
		graphics.fillOval(x, y, width, height);
	}
	
	/**
	 * {@inheritDoc}
	 */
	public void setColor(Color color) {
		graphics.setColor(color);
	}

	/**
	 * {@inheritDoc}
	 */
	public void setLineWidth(float lineWidth) {
		graphics.setStroke(new BasicStroke(lineWidth));
	}

	/**
	 * {@inheritDoc}
	 */
	public void drawImage(BufferedImage image, int x, int y, int width,
			int height) {
		graphics.drawImage(image, x, y, width, height, null);		
	}
}
