package org.helioviewer.viewmodel.view;

import java.util.AbstractList;
import java.util.LinkedList;
import java.util.concurrent.locks.ReentrantLock;

import org.helioviewer.viewmodel.changeevent.ChangeEvent;

/** Abstract base class implementing View, managing view listeners.
 * 
 * <p>This class provides the functionality to manage and notify all
 * view listeners.
 * 
 * <p>For further information about views, see {@link View}.
 * 
 * @author Markus Langenberg
 */
public abstract class AbstractView implements View {

	private AbstractList<ViewListener> listeners = new LinkedList<ViewListener>();
	private ReentrantLock lock = new ReentrantLock();
    
	/**
	 * {@inheritDoc}
	 */
	public void addViewListener(ViewListener l) {
		lock.lock();
		listeners.add(l);
		lock.unlock();
	}

	/**
	 * {@inheritDoc}
	 */
	public AbstractList<ViewListener> getAllViewListener() {
		return listeners;
	}

	/**
	 * {@inheritDoc}
	 */
	public void removeViewListener(ViewListener l) {
		lock.lock();
		listeners.remove(l);
		lock.unlock();
	}
	
	/** 
	 * Sends a new ChangeEvent to all registered view listeners.
     * 
     * @param aEvent ChangeEvent to send
     */
    protected void notifyViewListeners(ChangeEvent aEvent) {
    	lock.lock();
    	
    	try {
	        for(ViewListener v : listeners) {
	        	v.viewChanged(this, aEvent);
	        }
    	} finally {
    		lock.unlock();
    	}
    }

}
