package org.helioviewer.viewmodel.view;

import org.helioviewer.viewmodel.changeevent.ChangeEvent;

/** Listener to react on changes within a view.
 * 
 * <p>This class provides the capability to catch events from
 * within the view chain. Therefore, the implementing class
 * does not necessarily have to be a view itself, other classes
 * working together with the view chain without being a view
 * themselves also are able to be a view listener. On the other
 * hand, since implementing this interface is the only way to
 * receives changes within the viewchain, every view except from
 * the undermost view should implement this interface.
 * 
 * <p>When a change takes place, a ChangeEvent is emitted, containing
 * informations about the cause and consequences of the change.
 * 
 * <p>For further information about change event, see
 * {@link org.helioviewer.viewmodel.changeevent}
 * 
 * @author Ludwig Schmidt
 *
 */
public interface ViewListener {
    
	/** Callback function that will be called on every change within a view or its predecessors.
	 * 
	 * @param sender View which emitted the ChangeEvent
	 * @param aEvent History of changes, containing the causes and views where changes took place.
	 */
    public void viewChanged(View sender, ChangeEvent aEvent);

}
