package org.helioviewer.viewmodel.view.bufferedimage;

import java.awt.Graphics2D;
import java.awt.image.BufferedImage;

import org.helioviewer.viewmodel.imagedata.ARGBInt32ImageData;
import org.helioviewer.viewmodel.imagedata.ImageData;
import org.helioviewer.viewmodel.imagedata.JavaBufferedImageData;
import org.helioviewer.viewmodel.view.AbstractLayeredView;
import org.helioviewer.viewmodel.view.SubimageDataView;
import org.helioviewer.viewmodel.view.View;
import org.helioviewer.viewmodel.view.ViewHelper;
import org.helioviewer.viewmodel.viewportimagesize.ViewportImageSize;

/** Implementation of LayeredView for rendering in software mode.
 * 
 * <p>This class merged multiple layers by drawing them into one
 * single image, including scaling and moving them in a correct way.
 * 
 * <p>For further information about the role of the LayeredView
 * within the view chain, see
 * {@link org.helioviewer.viewmodel.view.LayeredView}
 * 
 * @author Ludwig Schmidt
 * @author Markus Langenberg
 */
public class BufferedImageLayeredView extends AbstractLayeredView implements SubimageDataView {

    ARGBInt32ImageData imageData;
    BufferedImage buffer;
    
    /**
     * {@inheritDoc}
     */
    public ImageData getSubimageData() {
        return imageData;
    }

    /**
     * {@inheritDoc}
     */
    protected void redrawBufferImpl() {
        
        if ((viewportImageSize == null) || (viewportImageSize.getWidth() <= 0) || 
        		(viewportImageSize.getHeight() <= 0)) {
                
        	return;
        }

        layerLock.lock();
        
        buffer = new BufferedImage(viewportImageSize.getWidth(),
                viewportImageSize.getHeight(), BufferedImage.TYPE_INT_ARGB);
        Graphics2D g = buffer.createGraphics();
        
        try {
	        for (View v : layers) {
	        	Layer layer = viewLookup.get(v);
	        	
	            if (!layer.visibility) {
	                continue;
	            }
	
	            ViewportImageSize s = ViewHelper.calculateViewportImageSize(
	            		layer.viewportView.getViewport(), layer.regionView.getRegion());
	
	            if (s == null || !s.hasArea()) {
	                continue;
	            }
	
	            JavaBufferedImageData data = ViewHelper.getImageDataAdapter(v,
	                    JavaBufferedImageData.class);
	
	            if (data == null) {
	                continue;
	            }
	            BufferedImage img = data.getBufferedImage();
	
	            g.drawImage(img, layer.renderOffset.getX(), layer.renderOffset.getY(), s.getWidth(), s
	                    .getHeight(), null);
	        }
        } finally {
        	layerLock.unlock();
        }
        imageData = new ARGBInt32ImageData(buffer);
    }
}
