package org.helioviewer.webbackend;

import java.awt.Graphics;

import org.helioviewer.base.math.Vector2dInt;
import org.helioviewer.viewmodel.changeevent.ChangeEvent;
import org.helioviewer.viewmodel.changeevent.SubImageDataChangedReason;
import org.helioviewer.viewmodel.imagedata.JavaBufferedImageData;
import org.helioviewer.viewmodel.region.Region;
import org.helioviewer.viewmodel.view.AbstractView;
import org.helioviewer.viewmodel.view.RegionView;
import org.helioviewer.viewmodel.view.View;
import org.helioviewer.viewmodel.view.ViewHelper;
import org.helioviewer.viewmodel.view.ViewListener;
import org.helioviewer.viewmodel.view.ViewportView;
import org.helioviewer.viewmodel.viewport.Viewport;
import org.helioviewer.viewmodel.viewportimagesize.ViewportImageSize;

public class BufferedImageJavaGraphicsRenderableView extends AbstractView 
		implements JavaGraphicsRenderableView, ViewListener {

    View innerView;
    RegionView regionView;
    
    JavaBufferedImageData imageData;
    
    public BufferedImageJavaGraphicsRenderableView(View toRender) {
        innerView = toRender;
        innerView.addViewListener(this);
        imageData = ViewHelper.getImageDataAdapter(innerView, JavaBufferedImageData.class);
    }

    public void render(Graphics g, Vector2dInt offset) {
        if(imageData != null) {
            Viewport v = ViewHelper.getViewAdapter(innerView, ViewportView.class).getViewport();
            Region r = ViewHelper.getViewAdapter(innerView, RegionView.class).getRegion();
            ViewportImageSize s = ViewHelper.calculateViewportImageSize(v, r);
            g.drawImage(imageData.getBufferedImage(), offset.getX(),
                    offset.getY(), s.getWidth(), s.getHeight(), null);
        }
    }

    public void render(Graphics g) {
        render(g, Vector2dInt.NULL_VECTOR);
    }

    public void viewChanged(View sender, ChangeEvent aEvent) {
        imageData = ViewHelper.getImageDataAdapter(innerView, JavaBufferedImageData.class);
        notifyViewListeners(aEvent);
    }

    public void triggerRedraw() {
        notifyViewListeners(new ChangeEvent(new SubImageDataChangedReason(this)));
    }

    @SuppressWarnings("unchecked")
	public <T extends View> T getAdapter(Class<T> c) {
        if (c.isInstance(this))
            return (T) this;
        else return innerView.getAdapter(c);
    }
}
